%******************************************************************************
% TITLE
%   Geophysical Fluid Dynamics - Computer Exercises -
%
%   Brownian Motion:
%   Lagrangian motion of particles governed by stochastic process
%
% HISTORY
%   1999/04/24 Manabu Kanao (1st grade of master course in Meteology Lab., 
%                            Division of Earth and Planetary Sciences,
%                            Graduate School of Science)
%   1999/04/24 Shigeo Yoden
%   2004/05/25 Koichi Shin (RISH) (convert to MATLAB code)
%
% INPUT
%   a0,      % magnitude of static external force
%   a1,      % magnitude of linear external force (Rayleigh friction coefficient)
%   b,       % magnitude of stochastic external force
%   iseed,   % seed for pseudo-random number (RANDLIB): iseed ~= 0
%******************************************************************************
function brown1(a0,a1,b,iseed)

  global xmin xmax ymin ymax mmax

  rand('state',iseed);

  % nmax: number of particle, mmax: number of figure
  nmax=6;
  mmax=10;

  % itmax: number of iteration
  itmax=400;
  dt=0.1;          % dt: temporal grid size

  xmin=-5.0;xmax=5.0;ymin=-5.0;ymax=5.0;

  h.fig = gcf;

  % set initial values
  x = zeros(itmax+1,nmax);
  y = zeros(itmax+1,nmax);

  %***************************
  % draw mmax figures
  for m=1:mmax

    uv = zeros(2,nmax);
    srand = zeros(2,nmax);
  
    % temporal development
    for l=1:itmax
      %srand = randn(1,2);
      for n=1:nmax
        srand(:,n) = rand_nr';
      end
    
      duv = -a1*uv + b*srand;
      duv(2,:) = duv(2,:)-a0;
    
      duvdt = duv*dt;
      
      uv = uv+duvdt;

      x(l+1,:) = x(l-1+1,:) + duvdt(1,:);
      y(l+1,:) = y(l-1+1,:) + duvdt(2,:);
    end

    %***************************
    % graph of trajectory
    h = graph(h,x,y,m);
  
    %***************************
    % pause
    set(gcf,'WindowButtonUpFcn','pushanykey')
    set(gcf,'KeyPressFcn','pushanykey')
    %set(gcf,'ButtonDownFcn','pushanykey')

    ud = get(gcf, 'UserData');
    if strcmp(ud, 'stop') || strcmp(ud, 'exit')
      return;
    end
    
    if m < mmax
        uiwait(gcf);
    end
  end

return
%*****************************************************************************

function h = graph(h,x,y,m)

  global xmin xmax ymin ymax mmax
  
  figure(h.fig)

  plot(x,y);

  axis equal;
  axis([xmin, xmax, ymin, ymax]);
  xlabel('X')
  ylabel('Y')
  set(gca,'XTick',xmin:2:xmax);
  set(gca,'YTick',ymin:2:ymax);

  title(sprintf('EXAMPLE %02d/%02d',m,mmax));
return
