function varargout = brown1_ui(varargin)
% BROWN1_UI Application M-file for brown1_ui.fig
%    FIG = BROWN1_UI launch brown1_ui GUI.
%    BROWN1_UI('callback_name', ...) invoke the named callback.

% Last Modified by GUIDE v2.5 10-Dec-2005 00:20:01

if nargin == 0  % LAUNCH GUI

  fig = openfig(mfilename,'reuse');
  %set(fig,'Color',get(0,'defaultUicontrolBackgroundColor'));
  changeuicolor(fig)

  % Generate a structure of handles to pass to callbacks, and store it. 
  handles = guihandles(fig);
  set_figstate('idle', handles);

  % STARTUP code ----
  [param.a0, param.a1, param.b, param.seed, param.nmax, param.step, param.label] = textread('expar12.data','%f%f%f%f%f%f%q%*[^\n]', 'commentstyle','matlab');
  set(handles.popup_sample,'String',param.label);
  
  handles.param = param;
  
  setparam(handles,1);
  % STARTUP code ----
  
  guidata(fig, handles);

  if nargout > 0
    varargout{1} = fig;
  end

else
  if ischar(varargin{1}) % INVOKE NAMED SUBFUNCTION OR CALLBACK
  try
    if (nargout)
      [varargout{1:nargout}] = feval(varargin{:}); % FEVAL switchyard
    else
      feval(varargin{:}); % FEVAL switchyard
    end
  catch
    disp(lasterr);
  end
  end
end


return;

%----
function setparam(handles,n)
  set(handles.edit1,'String',handles.param.a0(n));
  set(handles.edit2,'String',handles.param.a1(n));
  set(handles.edit3,'String',handles.param.b(n));
  set(handles.edit4,'String',handles.param.seed(n));
return;


% --------------------------------------------------------------------
function varargout = popup_sample_Callback(h, eventdata, handles, varargin)
  n = get(h,'Value');
  setparam(handles,n);
return;


% --------------------------------------------------------------------
% --------------------------------------------------------------------
% --------------------------------------------------------------------
function set_figstate(newstate, handles)
set(gcf, 'UserData', newstate);
ud = get(gcf, 'UserData');
switch ud
    case 'idle' % ready to change parameters, start calculation or exit
        set(findobj(gcf, 'Type', 'uicontrol'), 'Enable', 'on');
        set(findobj(gcf, 'Type', 'uicontrol', 'Style', 'edit'), 'Enable', 'on');
        set(findobj(gcf, 'Type', 'uicontrol', 'Style', 'edit'), 'BackgroundColor', [1.0 1.0 1.0]);
        set(handles.start, 'String', '[> START');
        set(handles.start, 'Enable', 'on');
        set(handles.stop, 'Enable', 'off');
        set(handles.exit, 'Enable', 'on');
    case 'run' % executing calculation
        set(findobj(gcf, 'Type', 'uicontrol'), 'Enable', 'on');
        set(findobj(gcf, 'Type', 'uicontrol', 'Style', 'popupmenu'), 'Enable', 'off');
        set(findobj(gcf, 'Type', 'uicontrol', 'Style', 'edit'), 'Enable', 'inactive');
        set(findobj(gcf, 'Type', 'uicontrol', 'Style', 'edit'), 'BackgroundColor', [0.8 0.8 0.8]);
        set(handles.start, 'String', '{}{} PAUSE');
        set(handles.start, 'Enable', 'on');
        set(handles.stop, 'Enable', 'on');
        set(handles.exit, 'Enable', 'off');
    case 'pause' % uiwait (waiting for uiresume)
        set(findobj(gcf, 'Type', 'uicontrol'), 'Enable', 'on');
        set(findobj(gcf, 'Type', 'uicontrol', 'Style', 'popupmenu'), 'Enable', 'off');
        set(findobj(gcf, 'Type', 'uicontrol', 'Style', 'edit'), 'Enable', 'inactive');
        set(findobj(gcf, 'Type', 'uicontrol', 'Style', 'edit'), 'BackgroundColor', [0.8 0.8 0.8]);
        set(handles.start, 'String', '[> NEXT'); % NOT RESUME (only for brown1)
        set(handles.start, 'Enable', 'on');
        set(handles.stop, 'Enable', 'on');
        set(handles.exit, 'Enable', 'off');
    case 'stop' % trying to stop calculation
        set(findobj(gcf, 'Type', 'uicontrol'), 'Enable', 'off');
    case 'exit' % trying to exit
        set(findobj(gcf, 'Type', 'uicontrol'), 'Enable', 'off');
    otherwise
        herr = errordlg(['Error: unknown FIGSTATE = [' ud ']']);
        uiwait(herr);
end
return;

% --------------------------------------------------------------------
function start_Callback(hObject, eventdata, handles)
ud = get(gcf, 'UserData');
switch ud
    case 'idle' % --> START
        set_figstate('pause', handles); % NOT run (only for brown1)
        %
        a0 = str2double(get(handles.edit1,'String'));
        a1 = str2double(get(handles.edit2,'String'));
        b = str2double(get(handles.edit3,'String'));
        seed = str2double(get(handles.edit4,'String'));
        if isnan(a0) | isnan(a1) | isnan(b) |isnan(seed)
          errordlg('Input parameters must be numerical characters.');
          return;
        end
        %
        brown1(a0,a1,b,seed);
        %
        ud = get(gcf, 'UserData');
        if strcmp(ud, 'exit')
            delete(gcf);
            return;
        end
        set_figstate('idle', handles);
    case 'run' % --> PAUSE
        % do nothing (only for brown1)
    case 'pause' % --> RESUME
        %set_figstate('pause', handles);
        uiresume(gcf);
    case 'stop' % --> do nothing
        return;
    case 'exit' % --> do nothing
        return;
    otherwise
        herr = errordlg(['Error: unknown FIGSTATE = [' ud ']']);
        uiwait(herr);
end
return;

% --------------------------------------------------------------------
function stop_Callback(hObject, eventdata, handles)
%set(gcf, 'Name', 'Linear Advection and Diffusion - Please wait for stopping...');
set_figstate('stop', handles);
uiresume(gcf);
return;

% --------------------------------------------------------------------
function exit_Callback(hObject, eventdata, handles)
ud = get(gcf, 'UserData');
if strcmp(ud, 'idle')
    delete(gcf);
    return;
else
    set_figstate('exit', handles);
    uiresume(gcf);
end
return;
