%-----------------------------------------------------
%   Thermal convection
%-----------------------------------------------------

%------------- Runge-Kutta ------------
function p = runge_kutta(p)
  %real, dimension(im0,p.jm0) :: dz1  , dz2  , dz3  , dz4
  %real, dimension(imx,p.jmx) :: dtmp1, dtmp2, dtmp3, dtmp4

  [dz1, dtmp1] = drv(p, p.zeta      , p.temp        );
  [dz2, dtmp2] = drv(p, p.zeta+dz1/2, p.temp+dtmp1/2);
  [dz3, dtmp3] = drv(p, p.zeta+dz2/2, p.temp+dtmp2/2);
  [dz4, dtmp4] = drv(p, p.zeta+dz3  , p.temp+dtmp3  );
     
  p.zeta = p.zeta +(dz1   +2*dz2   +2*dz3   +dz4  ) /6;
  p.temp = p.temp +(dtmp1 +2*dtmp2 +2*dtmp3 +dtmp4) /6;

  p.p_time = p.p_time +p.dt;

return;
  
%------------- Vorticity Eq. ------------
function [dz, dtmp] = drv(p, zz, tt)
  %real, dimension(p.im0,p.jm0) :: zz  , dz
  %real, dimension(p.imx,p.jmx) :: tt, dtmp
  %real :: u(p.im1, p.jmx), v(p.imx, p.jm1)

  psi = poisson(p.fac, zz);

  u = -d_z(psi, p.dz);
  w =  d_x(psi, p.dx);

  %zzex = average(expand(expand(zz,0,1),0,2),2);
  %zzez = average(expand(expand(zz,0,2),0,1),1); 
  zzex = expand(zz,0,1);
  zzez = expand(zz,0,2); 

  dz = p.dt* ( -average(d_x(average(u,2).*zzex ,p.dx),1) ... 
               -average(d_z(average(w,1).*zzez ,p.dz),2) ...
               +p.alpha*d_x(average(tt,2),p.dx) ...
               +p.nu*( +d_x(d_x(zzex,p.dx),p.dx) ...
                       +d_z(d_z(zzez,p.dz),p.dz) ));

  ttex = expand(tt, 1,1);
  ttez = expand(tt,-1,2);
  
  dtmp = p.dt*( -d_x(u.*average(ttex, 1) ,p.dx) ... 
                -d_z(w.*average(ttez, 2) ,p.dz) ...
                -p.beta*average(w,2) ...
                +p.kappa*( +d_x(d_x(ttex, p.dx), p.dx) ...
                           +d_z(d_z(ttez, p.dz), p.dz) ));

return;

%--------------- derivative_x ------------------
function ret = d_x(var, d, dim)
  dim = 1;
  m  = size(var,dim);
  ret = (var(2:m,:) - var(1:(m-1),:))/d;
return
  
%--------------- derivative_z ------------------
function ret = d_z(var, d, dim)
  dim = 2;
  m  = size(var,dim);
  ret = (var(:,2:m) - var(:,1:(m-1)))/d;
return    
