%******************************************************************************
% TITLE
%   Geophysical Fluid Dynamics - Computer Exercises -
%
%   Fluid parcel in steady flow from Lagrange view (3)
%   Meandering jet
%
% HISTORY
%   1999/05/10 Tomonori Ohmiya
%   1999/05/11 Shigeo Yoden
%   2004/05/24 Koichi Shin (RISH) (convert to MATLAB code)
%
% INPUT
%     px,      % x-coordinate of initial circle
%     py,      % y-coordinate of initial circle
%     pr,      % radius of initial circle
%     u0,      % uniform velocity of background
%     dt
%     itmax
%******************************************************************************
function deform3(px,py,pr,uu0,varargin)

global xmin xmax ymin ymax nx ny dt
global amp u0
u0= uu0;

% nmax: number of particle, itmax: number of iteration
nmax=480;
itmax=30;

% dt: tempral grid size(=tend-t), istep: number of time step
dt=0.5;
istep=1;

amp = 1.0;


%**********
% Argument processing
for k = 1:length(varargin)
  if isempty(varargin{k})
    continue
  end
  switch k 
   case 1
    dt = varargin{1};
   case 2
    itmax = varargin{2};
  end
end



% ----------------------
xmin=0.;xmax=2*pi;
ymin=0.;ymax=xmax/2;

nx=41;ny=21;
nx1=17;ny1=9;
nx2= 5;ny2=3;


work = zeros(2,3);
x = zeros(nmax+1,itmax+1);
y = zeros(nmax+1,itmax+1);
u = zeros(nx,ny);
v = zeros(nx,ny);

%***************************
% flow field (u,v)
delx = (xmax-xmin)/(nx-1);
dely = (ymax-ymin)/(ny-1);

for j=1:ny
  for i=1:nx
    xx = xmin + delx*(i-1);
    yy = ymin + dely*(j-1);
    u(i,j) = - amp*sin(xx)*cos(yy) + u0;
    v(i,j) =   amp*cos(xx)*sin(yy);
  end
end

%***************************
% initial condition
t = 0.0;

n=(0:nmax)';
theta = n./nmax*2.0*pi;
x(n+1,0+1) = pr.*cos(theta) + px;
y(n+1,0+1) = pr.*sin(theta) + py;


%***************************
% time-integration
xy = zeros(2,nmax+1,itmax+1);
xy(1,:,:) = x;
xy(2,:,:) = y;

h = graphics_init(xy,u,v);

tend = t + dt;
for ii=1:itmax
  ud = get(gcf, 'UserData');
  if strcmp(ud, 'stop') || strcmp(ud, 'exit')
    return;
  end

  xy(:,:,ii+1) = odrkdu(2,'odrkg','fcn34', t, tend, istep, xy(:,:,ii));

  h = graphics(xy,ii,h);
end


%***************************
% graph
function h = graphics_init(xy,u,v)

global xmin xmax ymin ymax nx ny dt

h.fig = gcf;
set(h.fig,'KeyPressFcn','pauseplot(gcbo)');

[xx,yy] = meshgrid(xmin:(xmax-xmin)/(nx-1):xmax,ymin:(ymax-ymin)/(ny-1):ymax);

quiver(xx,yy,u',v','c')

axis equal
axis([xmin, xmax, ymin, ymax])
xlabel('X');
ylabel('Y');

str={'0','p/4','p/2','3p/4','p','5p/4','3p/2','7p/4','2p'};
set(gca,'fontname','symbol','xtick',[0:pi/4:2*pi],'xticklabel',str)
set(gca,'fontname','symbol','ytick',[0:pi/4:2*pi],'yticklabel',str)

hold on
h.plot = plot(xy(1,:,1), xy(2,:,1),'r.');
hold off

ctime = sprintf('T= %4.1f', 0);
h.title = title(ctime,'fontname','Default');

return;


function h = graphics(xy,n,h)
global xmin xmax ymin ymax nx ny dt
  set(h.plot,'LineStyle','-','Marker','none','Color', [1 0.5 0.5]);
  hold on 
  h.plot = plot(xy(1,:,n+1), xy(2,:,n+1),'r.');
  hold off
  ctime = sprintf('T= %4.1f', n*dt);
  set(h.title,'String',ctime)
drawnow
return
%****************************************************************************
