function varargout = deform3_ui(varargin)
% deform3_UI Application M-file for deform3_ui.fig
%    FIG = deform3_UI launch deform3_ui GUI.
%    deform3_UI('callback_name', ...) invoke the named callback.

% Last Modified by GUIDE v2.5 14-Dec-2005 23:12:39

if nargin == 0  % LAUNCH GUI

  fig = openfig(mfilename,'reuse');
  %set(fig,'Color',get(0,'defaultUicontrolBackgroundColor'));
  changeuicolor(fig)

  % Generate a structure of handles to pass to callbacks, and store it. 
  handles = guihandles(fig);

  % STARTUP code ----
  set(gcf, 'HandleVisibility', 'on');
  set(gcf, 'CloseRequestFcn', 'deform3_ui(''exit_Callback'',gcf,[],guidata(gcf))');
  set_figstate('idle', handles);

  param = [];
  paramfile = 'expar3.data';
  try
    [param.x, param.y, param.r, param.u0, param.label] = textread(paramfile,'%f%f%f%f%q%*[^\n]', 'commentstyle','matlab');
    set(handles.popup_sample,'String',param.label);
  catch
    herr = errordlg(['Cant read a example parameters file: ', ...
                     paramfile,'.'],'Error');
    uiwait(herr)
  end
  
  handles.param = param;
  setparam(handles,1);
  % STARTUP code ----

  guidata(fig, handles);

  if nargout > 0
    varargout{1} = fig;
  end

else
  if ischar(varargin{1}) % INVOKE NAMED SUBFUNCTION OR CALLBACK
    try
      if (nargout)
        [varargout{1:nargout}] = feval(varargin{:}); % FEVAL switchyard
      else
        feval(varargin{:}); % FEVAL switchyard
      end
    catch
      disp(lasterr);
    end
  end
  return
end

graph0(handles);

return

% --------------------------------------------------------------------
function graph0(handles)

% If the program is running or pausing or trying to stop calculation
ud = get(gcf, 'UserData');
if ~strcmp(ud, 'idle')
  return
end

% load parameters from GUI
p = getparams(handles);
if p.err
  return
end

deform3(p.px,p.py,p.pr,p.u0,p.dt,0);

return;

% --------------------------------------------------------------------
% load values from EDIT BOXES
function p = getparams(handles)

p.err = 0;

p.px = str2double(get(handles.edit1,'String'));
p.py = str2double(get(handles.edit2,'String'));
p.pr = str2double(get(handles.edit3,'String'));
p.u0 = str2double(get(handles.edit4,'String'));

p.dt = str2double(get(handles.edit5,'String'));
p.nmax = str2double(get(handles.edit6,'String'));

%
if isnan(p.px) | isnan(p.py) | isnan(p.pr) | isnan(p.u0)
  errordlg('Input parameters must be numerical characters.');
  p.err = 1;
end

return


% --------------------------------------------------------------------
function setparam(handles,n)

if ~isempty(handles.param)
  set(handles.edit1,'String',handles.param.x(n));
  set(handles.edit2,'String',handles.param.y(n));
  set(handles.edit3,'String',handles.param.r(n));
  set(handles.edit4,'String',handles.param.u0(n));
end

return;


% --------------------------------------------------------------------
function varargout = popup_sample_Callback(h, eventdata, handles, varargin)
n = get(h,'Value');
setparam(handles,n);
graph0(handles)

return;

% --------------------------------------------------------------------
% --------------------------------------------------------------------
% --------------------------------------------------------------------
function set_figstate(newstate, handles)
set(gcf, 'UserData', newstate);
ud = get(gcf, 'UserData');
switch ud
    case 'idle' % ready to change parameters, start calculation or exit
        set(findobj(gcf, 'Type', 'uicontrol'), 'Enable', 'on');
        set(findobj(gcf, 'Type', 'uicontrol', 'Style', 'edit'), 'Enable', 'on');
        set(findobj(gcf, 'Type', 'uicontrol', 'Style', 'edit'), 'BackgroundColor', [1.0 1.0 1.0]);
        set(handles.start, 'String', '[> START');
        set(handles.start, 'Enable', 'on');
        set(handles.stop, 'Enable', 'off');
        set(handles.exit, 'Enable', 'on');
    case 'run' % executing calculation
        set(findobj(gcf, 'Type', 'uicontrol'), 'Enable', 'on');
        set(findobj(gcf, 'Type', 'uicontrol', 'Style', 'popupmenu'), 'Enable', 'off');
        set(findobj(gcf, 'Type', 'uicontrol', 'Style', 'edit'), 'Enable', 'inactive');
        set(findobj(gcf, 'Type', 'uicontrol', 'Style', 'edit'), 'BackgroundColor', [0.8 0.8 0.8]);
        set(handles.start, 'String', '{}{} PAUSE');
        set(handles.start, 'Enable', 'on');
        set(handles.stop, 'Enable', 'on');
        set(handles.exit, 'Enable', 'off');
    case 'pause' % uiwait (waiting for uiresume)
        set(findobj(gcf, 'Type', 'uicontrol'), 'Enable', 'on');
        set(findobj(gcf, 'Type', 'uicontrol', 'Style', 'popupmenu'), 'Enable', 'off');
        set(findobj(gcf, 'Type', 'uicontrol', 'Style', 'edit'), 'Enable', 'inactive');
        set(findobj(gcf, 'Type', 'uicontrol', 'Style', 'edit'), 'BackgroundColor', [0.8 0.8 0.8]);
        set(handles.start, 'String', '[>RESUME');
        set(handles.start, 'Enable', 'on');
        set(handles.stop, 'Enable', 'on');
        set(handles.exit, 'Enable', 'off');
    case 'stop' % trying to stop calculation
        set(findobj(gcf, 'Type', 'uicontrol'), 'Enable', 'off');
    case 'exit' % trying to exit
        set(findobj(gcf, 'Type', 'uicontrol'), 'Enable', 'off');
    otherwise
        herr = errordlg(['Error: unknown FIGSTATE = [' ud ']']);
        uiwait(herr);
end
return;

% --------------------------------------------------------------------
function start_Callback(hObject, eventdata, handles)
ud = get(gcf, 'UserData');
switch ud
    case 'idle' % --> START
        set_figstate('run', handles);
        p = getparams(handles); % load parameters set in GUI
        if p.err
          return;
        end
        deform3(p.px,p.py,p.pr,p.u0,p.dt,p.nmax);
        ud = get(gcf, 'UserData');
        if strcmp(ud, 'exit')
            delete(gcf);
            return;
        end
        set_figstate('idle', handles);
    case 'run' % --> PAUSE
        set_figstate('pause', handles);
        uiwait(gcf);
    case 'pause' % --> RESUME
        set_figstate('run', handles);
        uiresume(gcf);
    case 'stop' % --> do nothing
        return;
    case 'exit' % --> do nothing
        return;
    otherwise
        herr = errordlg(['Error: unknown FIGSTATE = [' ud ']']);
        uiwait(herr);
end
return;

% --------------------------------------------------------------------
function stop_Callback(hObject, eventdata, handles)
%set(gcf, 'Name', 'Linear Advection and Diffusion - Please wait for stopping...');
set_figstate('stop', handles);
uiresume(gcf);
return;

% --------------------------------------------------------------------
function exit_Callback(hObject, eventdata, handles)
ud = get(gcf, 'UserData');
if strcmp(ud, 'idle')
    delete(gcf);
    return;
else
    set_figstate('exit', handles);
    uiresume(gcf);
end
return;


% --------------------------------------------------------------------
function edit1_Callback(hObject, eventdata, handles)
function edit2_Callback(hObject, eventdata, handles)
function edit3_Callback(hObject, eventdata, handles)
function edit4_Callback(hObject, eventdata, handles)
function edit5_Callback(hObject, eventdata, handles)
function edit6_Callback(hObject, eventdata, handles)
