%******************************************************************************
% TITLE
%   Geophysical Fluid Dynamics - Computer Exercises -
%
%   Fluid parcel in steady flow from Lagrange view (3)
%   Meandering jet + checkered flag
%
% HISTORY
%   1999/05/10 Tomonori Ohmiya
%   1999/05/11 Shigeo Yoden
%   2004/05/24 Koichi Shin (RISH) (convert to MATLAB code)
%
% INPUT
%     xoffst,      % x-coodinate of initial center of flag
%     yoffst,      % y-coodinate of initial center of flag
%     rlen,        % length of edge of initial flag
%     u0,          % uniform velocity of background
%     dt
%     itmax
%******************************************************************************
function deform4(xoffst, yoffst, rlen, uu0, varargin)

global xmin xmax ymin ymax nx ny dt
global amp u0
u0 = uu0;

nmax=248;
itmax=5;   % nmax: number of particles, itmax: number of iteration
dt=1.2;
istep=1;     % dt: temporal grid size(=tend-t), istep: number of step
kmax=8;

xmin=0.0;
xmax=2*pi;
ymin=0.0;
ymax=xmax/2;

nx=41;ny=21;
nx1=17;ny1=9;
nx2= 5;ny2=3;

amp = 1.0;


%**********
% Argument processing
for k = 1:length(varargin)
  if isempty(varargin{k})
    continue
  end
  switch k 
   case 1
    dt = varargin{1};
   case 2
    itmax = varargin{2};
  end
end

work = zeros(2,3);

%
x = zeros(nmax+1,kmax+1,itmax+1);
y = zeros(nmax+1,kmax+1,itmax+1);
u = zeros(nx,ny);
v = zeros(nx,ny);


%***************************
% flow field (u,v)
delx = (xmax-xmin)/(nx-1);
dely = (ymax-ymin)/(ny-1);

for j=1:ny;
  for i=1:nx;
    xx = xmin + delx*(i-1);
    yy = ymin + dely*(j-1);
    u(i,j) = - amp*sin(xx)*cos(yy) + u0;
    v(i,j) =   amp*cos(xx)*sin(yy);
  end
end

%***************************
% initial condition
t = 0.0;

xoff(0+1) = xoffst  - rlen/2;
xoff(1+1) = xoff(0+1);
xoff(2+1) = xoff(1+1) + rlen/2;
xoff(3+1) = xoff(0+1) + rlen/4;
xoff(4+1) = xoff(3+1) + rlen/2;
xoff(5+1) = xoff(0+1);
xoff(6+1) = xoff(5+1) + rlen/2;
xoff(7+1) = xoff(0+1) + rlen/4;
xoff(8+1) = xoff(7+1) + rlen/2;

yoff(0+1) = yoffst  - rlen/2;
yoff(1+1) = yoff(0+1);
yoff(2+1) = yoff(1+1);
yoff(3+1) = yoff(0+1) + rlen/4;
yoff(4+1) = yoff(3+1);
yoff(5+1) = yoff(0+1) + rlen/2;
yoff(6+1) = yoff(5+1);
yoff(7+1) = yoff(0+1) + rlen*3/4;
yoff(8+1) = yoff(7+1);

for k=0:kmax
  if k == 0
    rl = rlen;
  else
    rl = rlen/4;
  end

  for n=1:(nmax/4)
    dx = rl/(nmax/4);
    x(n         +1,k+1,0+1) =        dx*(n-1) + xoff(k+1);
    x(n+  nmax/4+1,k+1,0+1) =   rl            + xoff(k+1);
    x(n+2*nmax/4+1,k+1,0+1) =   rl - dx*(n-1) + xoff(k+1);
    x(n+3*nmax/4+1,k+1,0+1) =                 + xoff(k+1);
    y(n         +1,k+1,0+1) =                 + yoff(k+1);
    y(n+  nmax/4+1,k+1,0+1) =        dx*(n-1) + yoff(k+1);
    y(n+2*nmax/4+1,k+1,0+1) =   rl            + yoff(k+1);
    y(n+3*nmax/4+1,k+1,0+1) =   rl - dx*(n-1) + yoff(k+1);
  end

  x(0+1,k+1,0+1) = x(nmax+1,k+1,0+1);
  y(0+1,k+1,0+1) = y(nmax+1,k+1,0+1);
end
  
%***************************
% time-integration

xy = zeros(2,nmax+1,kmax+1,itmax+1);
xy(1,:,:,:) = x;
xy(2,:,:,:) = y;

h = graphics_init(xy,u,v);

for ii=1:itmax
  ud = get(gcf, 'UserData');
  if strcmp(ud, 'stop') || strcmp(ud, 'exit')
    return;
  end

  tend = t + dt;
  for k=0:kmax
    xy(:,:,k+1,ii+1) = odrkdu(2,'odrkg','fcn34',t,tend,istep,xy(:,:,k+1,ii));
  end 

  h = graphics(xy,ii,h);
end

%x(:,:,:)= xy(1,:,:,:);
%y(:,:,:)= xy(2,:,:,:);
return;

%***************************
% graph
function h = graphics_init(xy,u,v)

global xmin xmax ymin ymax nx ny dt

h.fig = gcf;
set(h.fig,'KeyPressFcn','pauseplot(gcbo)');
[xx,yy] = meshgrid(xmin:((xmax-xmin)/(nx-1)):xmax, ymin:((ymax-ymin)/(ny-1)):ymax);
quiver(xx,yy,u',v','c')

axis equal
axis([xmin, xmax, ymin, ymax])
xlabel('X');
ylabel('Y');

str={'0','p/4','p/2','3p/4','p','5p/4','3p/2','7p/4','2p'};
set(gca,'fontname','symbol','xtick',[0:pi/4:2*pi],'xticklabel',str)
set(gca,'fontname','symbol','ytick',[0:pi/4:2*pi],'yticklabel',str)

x(:,:,:)= xy(1,:,:,:);
y(:,:,:)= xy(2,:,:,:);
hold on
plot(xy(1,:,1,1),xy(2,:,1,1),'b-'); % outer frame
%fill(xy(1,:,2:end,1),xy(2,:,2:end,1),'b');
fill(x(:,2:end,1),y(:,2:end,1),'b');
hold off

h.title = title(sprintf('Time= %4.1f',0),'fontname','Default');
drawnow

return

function h = graphics(xy,n,h)
global xmin xmax ymin ymax nx ny dt

x(:,:,:)= xy(1,:,:,:);
y(:,:,:)= xy(2,:,:,:);
hold on
plot(xy(1,:,1,n+1),xy(2,:,1,n+1),'b-'); % outer frame
fill(x(:,2:end,n+1),y(:,2:end,n+1),'b');
hold off

set(h.title,'String',sprintf('Time= %4.1f',n*dt))

drawnow
return

%*****************************************************************************
