%*-----------------------------------------------------------------------
%*     Runge-Kutta-Gill algorithm routine.
%*                                                 Oct. 5, 1990  S.Sakai
%*-----------------------------------------------------------------------
%*     Copyright (C) 2000 GFD Dennou Club. All rights reserved.
%*-----------------------------------------------------------------------
%@converted for 2-D arrays
function xout = odrkg(n, fcn, t, dt, x, dx)
      %dimension  x(n), dx(n), xout(n), work(n)

      work = zeros(size(x));
      
      dt2 = dt/2.;
      c1 =  (1. - 1./sqrt(2.))*dt;
      c2 =  -2. + 3./sqrt(2.);
      c3 =   2. -    sqrt(2.);
      c4 =  (1. + 1./sqrt(2.))*dt;
      c5 =  (2. + 3./sqrt(2.))/3.;
      c6 = -(2. +    sqrt(2.))/3.;

      %*--------------------------- 1st step ----------------------------------
      xout = x + dt2*dx;

      %*--------------------------- 2nd step ----------------------------------
      tt = t+dt2;
      work = feval(fcn, n, tt, xout, work);
      
      xout = xout  + c1*(work-dx);
      dx   = c2*dx + c3*work;
      

      %*--------------------------- 3rd step ----------------------------------
      work = feval(fcn, n, tt, xout, work);
      
      xout = xout  + c4*(work-dx);
      dx   = c5*dx + c6*work;

      %*--------------------------- 4th step ----------------------------------

      tt = t+dt;
      %call fcn(n, tt, xout, work)
      work = feval(fcn, n, tt, xout, work);
      
      xout = xout + dt*(work/6.0+dx);

return
