%-----------------------------------------------------------------------
%      Ekman Spiral
%
%rho=1.0;
%eta=1.0;
%tau = 1.0;
%f   = 1.0;
%-----------------------------------------------------------------------
function ekman(ty, ny, f, finit)

nmax=50;
h=20.0;
dz = h/(nmax-0.5);

%ty = tau/rho; 
%ny = eta/rho;
%f;

%
D = sqrt(2*ny/abs(f));

%  ---- initializing variables ----
ii=(1:nmax)';
z = (ii-0.5)*dz;
u = zeros(nmax,1);
v = zeros(nmax,1);

fac = 0.5;
dt = fac*(dz^2)/ny;
dt = 0.05;
m = 1;


%  ---- time integration ----
global zmax
zmax = 20;
Z = 0:0.2:zmax;

%U =  (D*ty)/(sqrt(2)*ny) .* cos((Z./D)+(pi/4)*(f/abs(f))) .*exp(-Z./D);
U =  (D*ty)/(sqrt(2)*ny) .* cos((Z./D)+(pi/4)) .*exp(-Z./D); % modified by S. OTSUKA(2005.12.20)

%V = -(D*ty)/(sqrt(2)*ny) .* sin((Z./D)+(pi/4)*(f/abs(f))) .*exp(-Z./D);
%V = -(D*ty)/(sqrt(2)*ny) .* sin((Z./D)+(pi/4)*(f/abs(f))) .*exp(-Z./D) .*(f/abs(f)); % modified according to S. Otsuka (2005/12/16) (by K. Sato)
%V = -(D*ty)/(sqrt(2)*ny) .* sin((Z./D)+(pi/4)) .*exp(-Z./D) .*(f/abs(f)); % modified by S. OTSUKA(2005.12.20)
V = -(D*ty)/(sqrt(2)*ny) .* sin((Z./D)+(pi/4)) .*exp(-Z./D) .*sign(f); % altered by K. Sato (2005.12.20)

% --
if finit == 0
    graph0(U,V,Z)
    return
  end

% --
handles = initgraph(U,V,Z);

x = U*0;
y = V*0;

l = line(x,y,Z,'color','r', 'Marker','.');

for t = 0:dt:20
  x = U*t;
  y = V*t;

  ud = get(gcf, 'UserData');
  if strcmp(ud, 'stop') || strcmp(ud, 'exit')
    return;
  end
  set(l,'xdata',x,'ydata',y);
  title(sprintf('Time = %5.2f, D = %g',t,D))
  drawnow
  rotate3d on
end

return;

% -------------------------------------------------------------------------
function handles = initgraph(U,V,Z)

global zmax
r = 10;

handles.fig = gcf;

%set(gcf,'doublebuffer','on')
set(gca,'nextplot','replacechildren')

plot3([0,0],[0,0],[0,zmax],'k-','LineWidth',2)
hold on
plot3([-r,r],[0,0],[zmax,zmax],'k-','LineWidth',2)
plot3([0,0],[-r,r],[zmax,zmax],'k-','LineWidth',2)

patch([r*0.9,r*0.9,r*1.2],[r/10,-r/10,0],[0,0,0],[1,0,0],'Clipping','off')
patch([r*0.9,r*0.9,r*1.2],[r/10,-r/10,0]+r/2,[0,0,0],[1,0,0],'Clipping','off')
patch([r*0.9,r*0.9,r*1.2],[r/10,-r/10,0]-r/2,[0,0,0],[1,0,0],'Clipping','off')
hold off

axis equal
axis([-r,r,-r,r,0,zmax])
xlabel('X')
ylabel('Y')
zlabel('Z')
set(gca,'ZDir','reverse')
grid on

return

% -------------------------------------------------------------------------
function graph0(U,V,Z)

global zmax
r = 1;


%set(gcf,'doublebuffer','on')
set(gca,'nextplot','replacechildren')

axis square
axis([-1,1,-1,1,0,zmax])


plot3(U,V,Z,'r.')
hold on
plot3([0,0],[0,0],[0,zmax],'k-','LineWidth',2)
plot3([-r,r],[0,0],[zmax,zmax],'k-','LineWidth',2)
plot3([0,0],[-r,r],[zmax,zmax],'k-','LineWidth',2)

a = zeros(size(Z));
quiver3(a,a,Z,U,V,a,0,'b.');


patch([r*0.9,r*0.9,r*1.2],[r/10,-r/10,0],[0,0,0],[1,0,0],'Clipping','off')
patch([r*0.9,r*0.9,r*1.2],[r/10,-r/10,0]+r/2,[0,0,0],[1,0,0],'Clipping','off')
patch([r*0.9,r*0.9,r*1.2],[r/10,-r/10,0]-r/2,[0,0,0],[1,0,0],'Clipping','off')
hold off

set(gca,'ZDir','reverse')
grid on

delete(get(gca,'title'))
delete(get(gca,'xlabel'))
delete(get(gca,'ylabel'))
delete(get(gca,'zlabel'))
title('Flow vector');
xlabel('U')
ylabel('V')
zlabel('Z')

return
