function varargout = ekman_ui(varargin)
% EKMAN_UI Application M-file for ekman_ui.fig
%    FIG = EKMAN_UI launch ekman_ui GUI.
%    EKMAN_UI('callback_name', ...) invoke the named callback.

% Last Modified by GUIDE v2.5 11-Dec-2005 13:39:14

if nargin == 0  % LAUNCH GUI
    %  disp('LAUNCH GUI')
    fig = openfig(mfilename,'reuse');

    % Generate a structure of handles to pass to callbacks, and store it. 
    handles = guihandles(fig);

    % STARTUP code ----
    set(gcf, 'HandleVisibility', 'on');
    set(gcf, 'CloseRequestFcn', 'ekman_ui(''exit_Callback'',gcf,[],guidata(gcf))');
    set_figstate('idle', handles);

    param = [];
    paramfile = 'expar1.data';
    try
      [param.ty, param.ny, param.f, param.label] = textread(paramfile,'%f%f%f%q%*[^\n]', 'commentstyle','matlab');
      set(handles.popupmenu1,'String',param.label);
    catch
      herr = errordlg(['Cant read a example parameters file: ', ...
                     paramfile,'.'],'Error');
      uiwait(herr)
    end

    handles.param = param;
    setparam(handles,1);

    % STARTUP code ----
    guidata(fig, handles);

    if nargout > 0
        varargout{1} = fig;
    end

else

  if ischar(varargin{1}) % INVOKE NAMED SUBFUNCTION OR CALLBACK
    %disp(varargin{1});
  try
    if (nargout)
      [varargout{1:nargout}] = feval(varargin{:}); % FEVAL switchyard
    else
      feval(varargin{:}); % FEVAL switchyard
    end
  catch
    disp(lasterr);
  end
  end
  return
end

%a = figure(fig);
%axes(handles.axes1)
run_ekman0(handles)
return;

% --------------------------------------------------------------------
function setparam(handles,n)

if ~isempty(handles.param)
  set(handles.edit1,'String',handles.param.ty(n));
  set(handles.edit2,'String',handles.param.ny(n));
  set(handles.edit4,'String',handles.param.f(n));
end

return;

% --------------------------------------------------------------------
function run_ekman(handles)

ty = str2double(get(handles.edit1,'String'));
ny = str2double(get(handles.edit2,'String'));
f = str2double(get(handles.edit4,'String'));

%
if isnan(ty) | isnan(ny) | isnan(f)
  errordlg('Input parameters must be numerical characters.');
  return;
end

ekman(ty,ny,f,1);
rotate3d on

return

% --------------------------------------------------------------------
function run_ekman0(handles)

ty = str2double(get(handles.edit1,'String'));
ny = str2double(get(handles.edit2,'String'));
f = str2double(get(handles.edit4,'String'));

%
if isnan(ty) | isnan(ny) | isnan(f)
  errordlg('Input parameters must be numerical characters.');
  return;
end

ekman(ty,ny,f,0);
rotate3d on

return

% --------------------------------------------------------------------
function varargout = edit1_Callback(h, eventdata, handles, varargin)
run_ekman0(handles)

% --------------------------------------------------------------------
function varargout = edit2_Callback(h, eventdata, handles, varargin)
run_ekman0(handles)

% --------------------------------------------------------------------
function varargout = edit3_Callback(h, eventdata, handles, varargin)
run_ekman0(handles)

% --------------------------------------------------------------------
function varargout = edit4_Callback(h, eventdata, handles, varargin)
run_ekman0(handles)

% -- Example ---------------------------------------------------------
function varargout = popupmenu1_Callback(h, eventdata, handles, varargin)

n = get(h,'Value');
setparam(handles,n);

run_ekman0(handles)

return;

% xy--------------------------------------------------------------------
function varargout = pushbutton4_Callback(h, eventdata, handles, varargin)
view(0,90.001)

% xz--------------------------------------------------------------------
function varargout = pushbutton5_Callback(h, eventdata, handles, varargin)
view(0,0)

% 3D--------------------------------------------------------------------
function varargout = pushbutton6_Callback(h, eventdata, handles, varargin)
view(-37.5,30)

% --------------------------------------------------------------------
function varargout = figure1_CreateFcn(h, eventdata, handles, varargin)
return

% --------------------------------------------------------------------
function varargout = axes1_CreateFcn(h, eventdata, handles, varargin)
return;


% --------------------------------------------------------------------
% --------------------------------------------------------------------
% --------------------------------------------------------------------
function set_figstate(newstate, handles)
set(gcf, 'UserData', newstate);
ud = get(gcf, 'UserData');
switch ud
    case 'idle' % ready to change parameters, start calculation or exit
        set(findobj(gcf, 'Type', 'uicontrol'), 'Enable', 'on');
        set(findobj(gcf, 'Type', 'uicontrol', 'Style', 'edit'), 'BackgroundColor', [1.0 1.0 1.0]);
        set(handles.start, 'String', '[> START');
        set(handles.start, 'Enable', 'on');
        set(handles.stop, 'Enable', 'off');
        set(handles.exit, 'Enable', 'on');
    case 'run' % executing calculation
        set(findobj(gcf, 'Type', 'uicontrol'), 'Enable', 'on');
        set(findobj(gcf, 'Type', 'uicontrol', 'Style', 'popupmenu'), 'Enable', 'off');
        set(findobj(gcf, 'Type', 'uicontrol', 'Style', 'edit'), 'Enable', 'inactive');
        set(findobj(gcf, 'Type', 'uicontrol', 'Style', 'edit'), 'BackgroundColor', [0.8 0.8 0.8]);
        set(handles.start, 'String', '{}{} PAUSE');
        set(handles.start, 'Enable', 'on');
        set(handles.stop, 'Enable', 'on');
        set(handles.exit, 'Enable', 'off');
    case 'pause' % uiwait (waiting for uiresume)
        set(findobj(gcf, 'Type', 'uicontrol'), 'Enable', 'on');
        set(findobj(gcf, 'Type', 'uicontrol', 'Style', 'popupmenu'), 'Enable', 'off');
        set(findobj(gcf, 'Type', 'uicontrol', 'Style', 'edit'), 'Enable', 'inactive');
        set(findobj(gcf, 'Type', 'uicontrol', 'Style', 'edit'), 'BackgroundColor', [0.8 0.8 0.8]);
        set(handles.start, 'String', '[>RESUME');
        set(handles.start, 'Enable', 'on');
        set(handles.stop, 'Enable', 'on');
        set(handles.exit, 'Enable', 'off');
    case 'stop' % trying to stop calculation
        set(findobj(gcf, 'Type', 'uicontrol'), 'Enable', 'off');
    case 'exit' % trying to exit
        set(findobj(gcf, 'Type', 'uicontrol'), 'Enable', 'off');
    otherwise
        herr = errordlg(['Error: unknown FIGSTATE = [' ud ']']);
        uiwait(herr);
end
return;

% --------------------------------------------------------------------
function start_Callback(hObject, eventdata, handles)
ud = get(gcf, 'UserData');
switch ud
    case 'idle' % --> START
        set_figstate('run', handles);
        run_ekman(handles)
        ud = get(gcf, 'UserData');
        if strcmp(ud, 'exit')
            delete(gcf);
            return;
        end
        set_figstate('idle', handles);
    case 'run' % --> PAUSE
        set_figstate('pause', handles);
        uiwait(gcf);
    case 'pause' % --> RESUME
        set_figstate('run', handles);
        uiresume(gcf);
    case 'stop' % --> do nothing
        return;
    case 'exit' % --> do nothing
        return;
    otherwise
        herr = errordlg(['Error: unknown FIGSTATE = [' ud ']']);
        uiwait(herr);
end
return;

% --------------------------------------------------------------------
function stop_Callback(hObject, eventdata, handles)
set_figstate('stop', handles);
uiresume(gcf);
return;

% --------------------------------------------------------------------
function exit_Callback(hObject, eventdata, handles)
ud = get(gcf, 'UserData');
if strcmp(ud, 'idle')
    delete(gcf);
    return;
else
    set_figstate('exit', handles);
    uiresume(gcf);
end
return;
