% geoadj Shallow water Equation
%
% AUTHOR
%   Sakai
%
% USAGE
%   geoadj(gg, ff, itype, amp, wnum, gtype)
%
% INPUT
%   gg    :
%   ff    :
%   itype :
%   amp   :
%   wnum  :
%   gtype :
%-----------------------------------------------------------------------
function geoadj(gg, ff, itype, amp, wnum, iitmax, nnmax, gtype)

  global f g dx dt h0 nmax itmax

  %gtype = 3
  %amp = 0.5;
  %itype = 1;
  %wnum = 5;

  nmax = nnmax;
  itmax = iitmax;
  %itmax = 150;
  %nmax=400;

  xmax=200.;

  dx=xmax/nmax;

  g=gg;
  f=ff;

  h0=1.0;

  h = zeros(nmax+1,1);
  xh = zeros(nmax+1,1);
  dh1 = zeros(nmax+1,1);
  dh2 = zeros(nmax+1,1);
  dh3 = zeros(nmax+1,1);
  dh4 = zeros(nmax+1,1);

  u = zeros(nmax,1);
  xu = zeros(nmax,1);
  du1 = zeros(nmax,1);
  du2 = zeros(nmax,1);
  du3 = zeros(nmax,1);
  du4 = zeros(nmax,1);

  v = zeros(nmax,1);
  dv1 = zeros(nmax,1);
  dv2 = zeros(nmax,1);
  dv3 = zeros(nmax,1);
  dv4 = zeros(nmax,1);

  % ---- initializing variables ----
  nw  = nmax/20;          % half of initial wave width
  nc  = nmax/2 + 1;       % index of center
  switch itype
   case 1 % sin
    dp  = pi/nw*wnum;
    h( (nc-nw):(nc+nw) ) = sin(dp*(-nw:nw))*amp ;
   case 2 % rectangle
    h( (nc-nw):(nc+nw) ) = amp ;
   otherwise
    error('itype')
  end

  xh = (0:nmax)*dx;
  xu = (0:(nmax-1))*dx+dx/2.0; % midpoint of xh

  fac = 1.0;
  dt = fac*dx/sqrt(h0*g);

  %ht = ones(itmax+1, nmax+1)*NaN;
  ht = ones(itmax+1, nmax+1)*(-inf);
  
  % --- 
  t=0.0;
  %set(gcf,'UserData',1);
  colormap(jet(256));
  
  % --- display initial values
  ii = 1;
  ht(ii,:) = h';
  handles = [];
  switch gtype
   case 1
     handles = gr1_init(xh,ht,xu,u,v,ii,t);
   case 3
     handles = gr3(xh,h,xu,u,v,ii,t, handles);
   otherwise
    error('gtype')
  end 

  %  ---- time integration ----
  %itmax
  for ii=2:(itmax+1)
    drawnow;
    [du1, dv1, dh1] = drv(u       , v,        h       );
    [du2, dv2, dh2] = drv(u+du1/2., v+dv1/2., h+dh1/2.);
    [du3, dv3, dh3] = drv(u+du2/2., v+dv2/2., h+dh2/2.);
    [du4, dv4, dh4] = drv(u+du3   , v+dv3   , h+dh3   );

    u = u + (du1 + du4 + (du2 + du3)*2)/6.;
    v = v + (dv1 + dv4 + (dv2 + dv3)*2)/6.;

    h = h + (dh1 + dh4 + (dh2 + dh3)*2)/6.;

    t = t+dt;

    ht(ii,:) = h';

    %-- 
    ud = get(gcf, 'UserData');
    if strcmp(ud, 'stop') || strcmp(ud, 'exit')
      return;
    end

    drawnow;
    %--- graphics
    switch gtype
     case 1
      handles = gr1(ht,u,v,ii,t,handles);
     case 3
      handles = gr3(xh,h,xu,u,v,ii,t,handles);
     otherwise
      error('gtype')
    end 
  end

return;

%-----------------------------------------------------------------------
function [du, dv, dh] = drv(u, v, h)

    global f g dx dt h0 nmax

    du = zeros(nmax,1);
    dv = zeros(nmax,1);
    dh = zeros(nmax+1,1);

    du         =  (f*v -g*d_x(h))*dt;
    dv         = -(f*u          )*dt;

    dh(1)      = -2*h0*u(1)   /dx*dt;    %  boundary condition at x=0
    dh(2:nmax) =   -h0* d_x(u)   *dt;
    dh(nmax+1) =  2*h0*u(nmax)/dx*dt;    %  boundary condition at x=xmax

return    
    
%---------------------- derivative_x ---------------------------------
function ret = d_x(var)
  global dx

  im  = size(var,1);
 
  ret = zeros(im-1,1);
  ret = (var(2:im) - var(1:(im-1)))/dx;

return
  
%-----------------------------------------------------------------------
% graphics
function graphics

return;


