%
%
% gwave(bv, omega, h=0.1, nstep=100, grid=128)
%  bv   : Brunt-Visla frequency
%  omega: forced frequency
%  h    : dt
%  nstep: maximum step number
%  grid : grid number
%
% ex.
%  bv=10;
%  omega=7;
%
function gwave(bv, omega, varargin)
  global im jm
  global cmin cmax
  
  %bv=10;    % Brunt-Visla frequency
  %omega=7;  % forced frequency
  h=0.05; %
  nstep= 100; %
  grid = 128; %

  for k = 1:length(varargin)
    arg = varargin{k};
    if isempty(arg)
      continue
    end
    switch k 
     case 1
      h = arg;
     case 2
      nstep = arg;
     case 3
      grid = arg;
    end
  end  
  
  im=grid;
  jm=grid;
  km=63;
  lm=63;
  
  i=1:im;
  j=1:jm;
  l=-lm:lm;
  k=-km:km;  
  x=(i-1)/im;
  y=(j-1)/jm;
  wj=ones(jm,1);
  wi=ones(1,im);

  %
  wl=ones(1,1+lm*2);
  wk=ones(1,1+km*2);
  ws=wl'*(k.*k)+(l.*l)'*wk;
  ws(lm+1,km+1)=1;
  ws=bv*(wl'*k)./sqrt(ws);

  %
  eps=1e-6;
  if(min(min(abs(ws-omega)))<eps) 
    set(gcf, 'Visible', 'off');
    h = errordlg('Resonance occurred. Please change omega slightly.');
    waitfor(h);
    set(gcf, 'Visible', 'on');
    return;
  end
  
  %
  x1=0.5;
  y1=0.5;
  sigma=0.02;
  g=exp(-(wj*((x-x1).*(x-x1))+((y-y1).*(y-y1))'*wi)/(2*sigma^2));
  zf=p2g2sa(lm,km,g);
  zf(1+lm,1+km)=0;

  %
  cmax = 1;
  cmin = -cmax;
  
  %  
  t=0;  
  z=zeros(1+lm*2,1+km*2);
  handles = m3grph(t,z);
  for istep=1:nstep
    t=h*istep;
    z=0.5*ws.*((sin(omega*t)+sin(ws*t))./(omega+ws)...
              -(sin(omega*t)-sin(ws*t))./(omega-ws)).*zf;
    
    ud = get(gcf, 'UserData');
    if strcmp(ud, 'stop') || strcmp(ud, 'exit')
      return;
    end
    drawnow;
    handles = m3grph(t,z);
  end
