function varargout = gwave_ui(varargin)
% Last Modified by GUIDE v2.5 15-Dec-2005 17:33:53

if nargin == 0  % LAUNCH GUI

  fig = openfig(mfilename,'reuse');
  %set(fig,'Color',get(0,'defaultUicontrolBackgroundColor'));
  %changeuicolor(fig)

  % Generate a structure of handles to pass to callbacks, and store it. 
  handles = guihandles(fig);

  % STARTUP code ----
  set(gcf, 'HandleVisibility', 'on');
  set(gcf, 'CloseRequestFcn', 'gwave_ui(''exit_Callback'',gcf,[],guidata(gcf))');
  set_figstate('idle', handles);

  param = [];
  paramfile = 'expar3.data';
  try
    [param.bv, param.omega, ...
     param.dt, param.nstep, param.grid, ...
     param.label1] = textread(paramfile,'%f%f%f%f%f%q%*[^\n]', 'commentstyle','matlab');
    set(handles.popup_sample,'String',param.label1);
  catch
    herr = errordlg(['Cant read a example parameters file: ', ...
                     paramfile,'.'],'Error');
    uiwait(herr)
  end

  param.itype = 1;
  
  handles.param = param;
  setparam(handles,1);
  % STARTUP code ----
  
  guidata(fig, handles);

  if nargout > 0
    varargout{1} = fig;
  end

else
  if ischar(varargin{1}) % INVOKE NAMED SUBFUNCTION OR CALLBACK
    try
      if (nargout)
        [varargout{1:nargout}] = feval(varargin{:}); % FEVAL switchyard
      else
        feval(varargin{:}); % FEVAL switchyard
      end
    catch
      disp(lasterr);
    end
  end
  return
end


init_graphics(handles)
return

% --------------------------------------------------------------------
% invoked to show initial values
function init_graphics(handles)

% If the program is running or pausing or trying to stop calculation
ud = get(gcf, 'UserData');
if ~strcmp(ud, 'idle')
  return
end

% To show initial state, call the calculation and drawing routines with
% nstep = 0.
nstep =get(handles.edit12,'String');
set(handles.edit12,'String',0);
start_Callback([], [], handles)
set(handles.edit12,'String',nstep);

return;


% --------------------------------------------------------------------
% get values from EDIT BOXES
function p = getparams(handles)

p.err = 0;

% get values from EDIT BOXES
p.bv = str2double(get(handles.edit1,'String'));
p.omega = str2double(get(handles.edit2,'String'));

p.dt = str2double(get(handles.edit11,'String'));
p.nstep = str2double(get(handles.edit12,'String'));
p.grid = str2double(get(handles.edit13,'String'));



% If a parameter is not numerical, return after showing an error message.
if isnan(p.bv) | isnan(p.omega)
  errordlg('Input parameters must be numerical characters.');
  p.err = 1;
end

return

% --------------------------------------------------------------------
function setparam(handles,n)

  if ~isempty(handles.param)
    set(handles.edit1,'String',handles.param.bv(n));
    set(handles.edit2,'String',handles.param.omega(n));
    set(handles.edit11,'String',handles.param.dt(n));
    set(handles.edit12,'String',handles.param.nstep(n));
    set(handles.edit13,'String',handles.param.grid(n));
  end

return;


% --------------------------------------------------------------------
function varargout = popup_sample_Callback(h, eventdata, handles, varargin)

n = get(h,'Value');
setparam(handles,n);
init_graphics(handles)

return;


% --------------------------------------------------------------------
function varargout = edit_Callback(h, eventdata, handles, varargin)
%init_graphics(handles)  % comment out by K. Sato 2005/12/14
return


% --------------------------------------------------------------------
% --------------------------------------------------------------------
% --------------------------------------------------------------------
function set_figstate(newstate, handles)
set(gcf, 'UserData', newstate);
ud = get(gcf, 'UserData');
switch ud
    case 'idle' % ready to change parameters, start calculation or exit
        set(findobj(gcf, 'Type', 'uicontrol'), 'Enable', 'on');
        set(findobj(gcf, 'Type', 'uicontrol', 'Style', 'edit'), 'BackgroundColor', [1.0 1.0 1.0]);
        set(handles.start, 'String', '[> START');
        set(handles.start, 'Enable', 'on');
        set(handles.stop, 'Enable', 'off');
        set(handles.exit, 'Enable', 'on');
    case 'run' % executing calculation
        set(findobj(gcf, 'Type', 'uicontrol'), 'Enable', 'on');
        set(findobj(gcf, 'Type', 'uicontrol', 'Style', 'popupmenu'), 'Enable', 'off');
        set(findobj(gcf, 'Type', 'uicontrol', 'Style', 'edit'), 'Enable', 'inactive');
        set(findobj(gcf, 'Type', 'uicontrol', 'Style', 'edit'), 'BackgroundColor', [0.8 0.8 0.8]);
        set(handles.start, 'String', '{}{} PAUSE');
        set(handles.start, 'Enable', 'on');
        set(handles.stop, 'Enable', 'on');
        set(handles.exit, 'Enable', 'off');
    case 'pause' % uiwait (waiting for uiresume)
        set(findobj(gcf, 'Type', 'uicontrol'), 'Enable', 'on');
        set(findobj(gcf, 'Type', 'uicontrol', 'Style', 'popupmenu'), 'Enable', 'off');
        set(findobj(gcf, 'Type', 'uicontrol', 'Style', 'edit'), 'Enable', 'inactive');
        set(findobj(gcf, 'Type', 'uicontrol', 'Style', 'edit'), 'BackgroundColor', [0.8 0.8 0.8]);
        set(handles.start, 'String', '[>RESUME');
        set(handles.start, 'Enable', 'on');
        set(handles.stop, 'Enable', 'on');
        set(handles.exit, 'Enable', 'off');
    case 'stop' % trying to stop calculation
        set(findobj(gcf, 'Type', 'uicontrol'), 'Enable', 'off');
    case 'exit' % trying to exit
        set(findobj(gcf, 'Type', 'uicontrol'), 'Enable', 'off');
    otherwise
        herr = errordlg(['Error: unknown FIGSTATE = [' ud ']']);
        uiwait(herr);
end
return;

% --------------------------------------------------------------------
function start_Callback(hObject, eventdata, handles)
ud = get(gcf, 'UserData');
switch ud
    case 'idle' % --> START
        set_figstate('run', handles);
        p = getparams(handles); % load parameters set in GUI
        if p.err
          return;
        end
        gwave(p.bv, p.omega, p.dt, p.nstep, p.grid);
        ud = get(gcf, 'UserData');
        if strcmp(ud, 'exit')
            delete(gcf);
            return;
        end
        set_figstate('idle', handles);
    case 'run' % --> PAUSE
        set_figstate('pause', handles);
        uiwait(gcf);
    case 'pause' % --> RESUME
        set_figstate('run', handles);
        uiresume(gcf);
    case 'stop' % --> do nothing
        return;
    case 'exit' % --> do nothing
        return;
    otherwise
        herr = errordlg(['Error: unknown FIGSTATE = [' ud ']']);
        uiwait(herr);
end
return;

% --------------------------------------------------------------------
function stop_Callback(hObject, eventdata, handles)
%set(gcf, 'Name', 'Linear Advection and Diffusion - Please wait for stopping...');
set_figstate('stop', handles);
uiresume(gcf);
return;

% --------------------------------------------------------------------
function exit_Callback(hObject, eventdata, handles)
ud = get(gcf, 'UserData');
if strcmp(ud, 'idle')
    delete(gcf);
    return;
else
    set_figstate('exit', handles);
    uiresume(gcf);
end
return;


% --------------------------------------------------------------------
function edit1_Callback(hObject, eventdata, handles)
function edit2_Callback(hObject, eventdata, handles)
function edit11_Callback(hObject, eventdata, handles)
function edit13_Callback(hObject, eventdata, handles)
function edit12_Callback(hObject, eventdata, handles)

