%******************************************************************************
% TITLE
%   Geophysical Fluid Dynamics - Computer Exercises -
%
%   Parcel Method (1): Inertial oscillation
%
% HISTORY
%   1999/06/06 Shigeo Yoden
%
% INPUT
%     phi, x0, y0, u0, v0
%******************************************************************************
function  inert1(phi,x0,y0,u0,v0, dt, nmax)

xmin=-500.;xmax=500.;ymin=-500.;ymax=500.;

mmax=1;                           % mmax: number of experiments to be displayed
%nmax=200;                         % nmax: number of time step
                                  % The unit of time is [day].


% Rotation system: the unit of length is [km].
x = (zeros(1,nmax+1));
y = (zeros(1,nmax+1));
% Inertial system: the unit of length is [km].
xi = (zeros(1,nmax+1));
yi = (zeros(1,nmax+1));
% Inertial system: the unit of length is [km].
xi0 = (zeros(1,nmax+1));
yi0 = (zeros(1,nmax+1));

%-----+--1---------2---------3---------4---------5---------6---------7-+

f  = 2.0 * (2.0*pi) * sin(phi*pi/180.0);

%dt = 0.01;
dt = dt / sin(phi*pi/180.0);

% change unit (s => day, m => km)
u0 = u0*86400/1000.0;                    % The unit of (u,v) is [km/day].
v0 = v0*86400/1000.0;

% temporal development
for n=1:(nmax+1)
  t=dt*(n-1);

  [x(n),y(n),xi(n),yi(n),xi0(n),yi0(n)] = integ(f, x0, y0, u0, v0, t);
end

%%%%%%%%%%%%%%%%%%%%%%%%
% graphic initialize
shg
hfig = gcf;
set(hfig,'DoubleBuffer','on')

% Erase axes in figure, unless the UserData exists (for GUI).
h = get(hfig,'Children');
for l = 1:length(h)
  if strcmp(get(h(l),'Type') ,'axes')
    if isempty(get(h(l),'UserData'))
      delete(h(l))
    end
  end
end

mainaxes = gca;
set(mainaxes,'Visible','off');
apos = get(mainaxes,'Position');
hax1 = axes('Position',[apos(1)          ,apos(2),apos(3)/2*0.8,apos(4)]);
hax2 = axes('Position',[apos(1)+apos(3)/2,apos(2),apos(3)/2*0.8,apos(4)]);

%%%%%%%%%%%%%%%%%%%%%%%
% animation
for n=0:nmax
  drawnow
  ud = get(gcf, 'UserData');
  if strcmp(ud, 'stop') || strcmp(ud, 'exit')
    return;
  end

  time  = n*dt;
  theta = f*time;

  %%%%%%%%%%%%%%%%%
  % rotating frame

  set(gcf, 'CurrentAxes', hax1);
  %axes(hax1)

  arrow(xmin,0,xmax-xmin,0,'k--');
  hold on
  arrow(0,ymin,0,ymax-ymin,'k-');
  plot(x(1),y(1),'r*')
  plot(x(1:(n+1)),y(1:(n+1)),'b-')
  plot(x(n+1),y(n+1),'mo')
  plot(x(1),y(1),'bd')
  hold off

  axis equal
  axis([xmin, xmax, ymin, ymax])
  xlabel('X [km]')
  ylabel('Y [km]')

  title(sprintf('T= %5.2f [day], f= %5.2e [/s]',time,f/86400))

  drawnow

  %%%%%%%%%%%%%%%%%
  % inertial frame
  set(gcf, 'CurrentAxes', hax2);
  %axes(hax2)

  xaxpx =  xmax*cos(0.5*theta);
  xaxpy =  xmax*sin(0.5*theta);
  yaxpx = -ymax*sin(0.5*theta);
  yaxpy =  ymax*cos(0.5*theta);

  arrow(-xaxpx, -xaxpy,2*xaxpx,2*xaxpy,'k--')
  hold on
  arrow(-yaxpx, -yaxpy,2*yaxpx,2*yaxpy,'k-')
  plot(xi(1:(n+1)),yi(1:(n+1)),'b-')
  plot(xi0(1:(n+1)),yi0(1:(n+1)),'r-')
  plot(xi0(n+1), yi0(n+1),'r*')
  plot(xi(n+1), yi(n+1),'mo')
  plot(xi(0+1), yi(0+1),'bd')
  hold off

  xlabel('X-inertial [km]')
  ylabel('Y-inertial [km]')
  axis equal
  axis([xmin, xmax, ymin, ymax])

  %
  drawnow
end


return;

      
%**********************************************************************
function [x,y,xi,yi,xi0,yi0] = integ(f, x0, y0, u0, v0, t)
theta = f*t;

x =   ( 1 - cos(theta) )*v0/f + u0*sin(theta)/f + x0;
y = - ( 1 - cos(theta) )*u0/f + v0*sin(theta)/f + y0;

xi  =  x*cos(0.5*theta) -  y*sin(0.5*theta);
yi  =  x*sin(0.5*theta) +  y*cos(0.5*theta);

xi0 = x0*cos(0.5*theta) - y0*sin(0.5*theta);
yi0 = x0*sin(0.5*theta) + y0*cos(0.5*theta);

return
%*****************************************************************************
