%******************************************************************************
% TITLE
%   Geophysical Fluid Dynamics - Computer Exercises -
%
%   Parcel Method (2): Inertial instability
%
% HISTORY
%   1999/06/03 Seiya Nishizawa
%   1999/06/04 Shigeo Yoden
%******************************************************************************
function  inert2(phi,p1,q1,p2,q2,p3,q3,dy, dt, nmax)

p = [p1,p2,p3];
q = [q1,q2,q3];

xmin=-100.;xmax=1900.;
ymin=0.;ymax=500.; % km/s
mmax=3;                           % mmax: number of experiment to be displayed
%nmax=200;                         % nmax: number of time step
jmax=10;                          % jmax: number of arrows of Ug
%dt=0.05;                          % dt:   time interval
                                  % The unit of time is [day].

%real x(0:nmax,mmax),y(0:nmax,mmax)          % The unit of length is [km].
x = zeros(nmax+1,mmax);
y = zeros(nmax+1,mmax);
%real p(mmax),q(mmax),ug(0:jmax,mmax)        % Horizontal shear p [m/s/1000km]
ug = zeros(jmax+1,mmax);

global x0 y0 v0
%-----+--1---------2---------3---------4---------5---------6---------7-+
% Load parameters
%      open(1,file='expar2.data')
%      read(1,*) phi, (p(m),q(m),m=1,mmax), dy, cdummy 
%      close(1)

x0  = 0.0;
y00 = (ymax-ymin)/2;
y0  = y00 + dy;
v0  = 0.0;
f   = 2.0 * (2.0*pi) * sin(phi*pi/180.0);

% mmax-th experiment
for m=1:mmax
  ugmin = q(m) - p(m)*ymax/1000;
  ugmax = q(m) + p(m)*ymax/1000;

  for j=0:jmax
    ug(j+1,m) = ugmin + (ugmax-ugmin)*j/jmax;
  end

  % change unit (s => day, m => km)
  p(m) = p(m)*8.64e-2;                    % 86400 x 10^-6 = 8.64 x 10^-2 
  q(m) = q(m)*86400/1000;                 % The unit of p and q becomes [1/day] and [km/day], respectively.

  u0   = p(m)*y00 + q(m) + f*dy;
  ugy0 = p(m)*y0  + q(m);
  % temporal development
  if f < p(m)
    for n=0:nmax
      t=dt*n;
      [x(n+1,m), y(n+1,m)] = integ1(f, p(m), u0, ugy0, t, x(n+1,m), y(n+1,m));
    end
  else
    if f == p(m)
      for n=0:nmax
        t=dt*n;
        [x(n+1,m), y(n+1,m)] =integ2(f, p(m), u0, ugy0, t, x(n+1,m), y(n+1,m));
      end
    else 
      for n=0:nmax
        t=dt*n;
        [x(n+1,m), y(n+1,m)] =integ3(f, p(m), u0, ugy0, t, x(n+1,m), y(n+1,m));
      end
    end
  end
end




%%%%%%%%%%%%%%%%%%%%%%%%
% graphic initialize
shg
hfig = gcf;
set(hfig,'DoubleBuffer','on')

% erase axes in figure, unless the UserData exists (for GUI).
h = get(gcf,'Children');
for l = 1:length(h)
  if strcmp(get(h(l),'Type') ,'axes')
    if isempty(get(h(l),'UserData'))
      delete(h(l))
    end
  end
end

mainaxes = gca;
set(mainaxes,'Visible','off');
apos = get(mainaxes,'Position');%left bottom w h

w1 = apos(3)*0.8;
w2 = apos(3)*0.2;
h = apos(4)/3;

hax(1) = axes('Position',[apos(1)+w2,apos(2)+h*2,w1,h*0.98]);
hax(2) = axes('Position',[apos(1)+w2,apos(2)+h,  w1,h*0.98]);
hax(3) = axes('Position',[apos(1)+w2,apos(2),    w1,h*0.98]);

hax(4) = axes('Position',[apos(1),apos(2)+h*2,w2*0.8,h*0.98]);
hax(5) = axes('Position',[apos(1),apos(2)+h,  w2*0.8,h*0.98]);
hax(6) = axes('Position',[apos(1),apos(2),    w2*0.8,h*0.98]);

umin=0.0;
umax=10.0;
for m=1:mmax

  % right
  axes(hax(m))
 
  hline(m) = plot(x(1:(n+1),m),y(1:(n+1),m),'m-');
  hold on
  hcircle(m) = plot(x(1,m), y(1,m),'ro');
  hold off

  axis equal 
  axis([xmin, xmax,  ymin,  ymax])

  switch m
   case 1
    set(gca,'XTickLabel','')
    htitle = title(sprintf('Lat.= deg., T= days'));
   case 2
    set(gca,'XTickLabel','')
   case 3
    xlabel('X [km]')
   otherwise
  end

  set(gca,'YTickLabel','')
  box on
  
  % left
  axes(hax(m+mmax))

  %call uxaxdv('b', (umax-umin)/4, umax-umin)

  hold on
  plot([ug(jmax+1,m),ug(0+1,m)], [ymax,ymin])
  
  for j=0:jmax
  %j=0:jmax;
    ya = ymin + (ymax-ymin)*j/jmax;
    %quiver(zeros(size(ya)),ya,ug(j+1,m),zeros(size(ya)),0,'c','filled')
    quiver(ones(size(ya)),ya,ug(j+1,m),ones(size(ya)),0,'c','filled')
  end
  hold off

  set(gca,'PlotBoxAspectRatio',[w2*0.8/w1*xmax/0.96 ymax 1]);
  axis([umin, umax, ymin, ymax])
  box on

  switch m
   case 1
    set(gca,'XTickLabel','')
   case 2
    set(gca,'XTickLabel','')
    ylabel('Y [km]')
   case 3
    xlabel('Ug[m/s]')
   otherwise
  end
  
end


%%%%%%%%%%%%%%
% animation
for n=1:nmax
  ud = get(gcf, 'UserData');
  if strcmp(ud, 'stop') || strcmp(ud, 'exit')
    return;
  end

  drawnow;
  
  %ct = sprintf('Lat.= %3.2f deg., T= %3.2f days', phi, n*dt);
  ct = sprintf('T= %3.2f days, f=%g', n*dt,f);
  set(htitle,'String',ct)

  for m=1:mmax
    set(hline(m),'xdata',x(1:(n+1),m),'ydata',y(1:(n+1),m));
    set(hcircle(m),'xdata',x(n+1,m),'ydata', y(n+1,m));
  end
  drawnow
end

return

%**********************************************************************


function [x, y] = integ1(f, p, u0, ugy0, t, x, y)

global x0 y0 v0

      %save
%-----+--1---------2---------3---------4---------5---------6---------7-+
      r = sqrt(f*(p-f));
      x =   0.5*(v0 - f*(u0-ugy0)/r) *exp( r*t) /(p-f) ...
       + 0.5*(v0 + f*(u0-ugy0)/r) *exp(-r*t) /(p-f) ...
                 + f*(u0-ugy0)*t /(p-f) ...
       + u0*t + x0 - v0/(p-f);
      y =   0.5*(v0 - f*(u0-ugy0)/r) *exp( r*t) /r ...
       - 0.5*(v0 + f*(u0-ugy0)/r) *exp(-r*t) /r ...
                 +   (u0-ugy0)/(p-f) ...
              + y0;

return

      
%*****************************************************************************
function [x,y] = integ2(f, p, u0, ugy0, t, x, y)
global x0 y0 v0

x = - (u0-ugy0)*t*(f*t)^2 /6 ...
    + 0.5* v0      *f*t^2 + u0*t + x0;
y = - 0.5*(u0-ugy0)*f*t^2 + v0*t + y0;

return

%*****************************************************************************
function [x,y] = integ3(f, p, u0, ugy0, t, x, y)
global x0 y0 v0

omega = sqrt( f*(f-p) );
x = (     ( 1 - cos(omega*t))*v0 ...
      - f*( t - sin(omega*t)/omega )*(u0 - ugy0) ) /(f-p) ...
      + u0*t + x0;
y =     - ( 1 - cos(omega*t))       *(u0 - ugy0)   /(f-p) ...
              + v0*sin(omega*t)/omega + y0;

return
%*****************************************************************************
