function varargout = linadv_ui(varargin)
% linadv_UI Application M-file for linadv_ui.fig
%    FIG = linadv_UI launch advection_linadv_ui GUI.
%    linadv_UI('callback_name', ...) invoke the named callback.
%
% 2004/05/24 Koichi Shin (RISH)
% 2005/12/08 Katsushi Sato
%

% Last Modified by GUIDE v2.5 13-Dec-2005 19:54:22

if nargin == 0  % LAUNCH GUI
  fig = openfig(mfilename,'reuse');

  % Generate a structure of handles to pass to callbacks, and store it. 
  handles = guihandles(fig);

  % Added STARTUP code ---- begin
  set(gcf, 'HandleVisibility', 'on');
  set(gcf, 'CloseRequestFcn', 'linadv_ui(''exit_Callback'',gcf,[],guidata(gcf))');
  set_figstate('idle', handles);
  
  try % for the case of loading error
    % reading example file
    param = [];
    paramfile = 'expar1.data'; % example file name
    [param.ijk, param.amp, param.rc, param.rnu, param.label] = textread(paramfile,'%f%f%f%f%q%*[^\n]', 'commentstyle','matlab');

    handles.param = param; % enable other functions to use the parameters

    % set the loaded parameters
    set(handles.popup_sample,'String',param.label);
    setparam(handles,1);

  catch
    % show popup window for loading error
    herr = errordlg(['Cant read a example parameters file: ', ...
                     paramfile,'.'],'Error');
    uiwait(herr);
  end
  % STARTUP code ---- end
  guidata(fig, handles); % assign handles to figure

  if nargout > 0
    varargout{1} = fig;
  end
  
else
  if ischar(varargin{1}) % INVOKE NAMED SUBFUNCTION OR CALLBACK
    try
      if (nargout)
        [varargout{1:nargout}] = feval(varargin{:}); % FEVAL switchyard
      else
        feval(varargin{:}); % FEVAL switchyard
      end
    catch
      disp(lasterr);
    end
  end
  return; % added to show initial values ("elseif" was separated into "else" and "if")
end

init_graph(handles); % show initial values
return;


% --------------------------------------------------------------------
% set parameters in EDIT BOXES
function setparam(handles,n)

if ~isempty(handles.param)
  set(handles.popup_ijk,'Value',handles.param.ijk(n));
  set(handles.edit_amp,'String',handles.param.amp(n));
  set(handles.edit_rc,'String',handles.param.rc(n));
  set(handles.edit_rnu,'String',handles.param.rnu(n));
end

return;

% --------------------------------------------------------------------
% get parameters from EDIT BOXES
function p = getparams(handles)

p.err = 0;

% get parameters from EDIT BOXES
p.ijk = get(handles.popup_ijk,'Value');
p.amp = str2double(get(handles.edit_amp,'String'));
p.rc = str2double(get(handles.edit_rc,'String'));
p.rnu = str2double(get(handles.edit_rnu,'String'));
p.nstep = str2double(get(handles.edit4,'String'));
p.twait = str2double(get(handles.edit5,'String'));

% If a parameter is not numerical, return after showing an error message.
if isnan(p.amp) | isnan(p.rc) | isnan(p.rnu)
  errordlg('Input parameters must be numerical characters.');
  p.err = 1;
end

return

% --------------------------------------------------------------------
% invoked to show initial values
function init_graph(handles)

% If the program is running or pausing or trying to stop calculation
ud = get(gcf, 'UserData');
if ~strcmp(ud, 'idle')
  return
end

% load the parameters set in GUI
p = getparams(handles);
if p.err
  return
end

% To show initial state, call the calculation and drawing routines with
% nstep = 0.
linadv(p.ijk,p.amp,p.rc,p.rnu,0,0);

return;


% --------------------------------------------------------------------
% invoked when the example is changed
% set example parameters and draw initial state
function popup_sample_Callback(hObject, eventdata, handles)
n = get(hObject,'Value');
setparam(handles,n);
init_graph(handles)
return;

% --------------------------------------------------------------------
% invoked when the type of initial function is changed
% (for drawing initial state)
function popup_ijk_Callback(hObject, eventdata, handles)
init_graph(handles)
return;

% --------------------------------------------------------------------
% invoked when the values of edit boxes are changed
% (for drawing initial state)
function varargout = edit_Callback(h, eventdata, handles, varargin)
init_graph(handles)
return;


% --------------------------------------------------------------------
% --------------------------------------------------------------------
% --------------------------------------------------------------------
function set_figstate(newstate, handles)
set(gcf, 'UserData', newstate);
ud = get(gcf, 'UserData');
switch ud
    case 'idle' % ready to change parameters, start calculation or exit
        set(findobj(gcf, 'Type', 'uicontrol'), 'Enable', 'on');
        set(findobj(gcf, 'Type', 'uicontrol', 'Style', 'edit'), 'BackgroundColor', [1.0 1.0 1.0]);
        set(handles.start, 'String', '[> START');
        set(handles.start, 'Enable', 'on');
        set(handles.stop, 'Enable', 'off');
        set(handles.exit, 'Enable', 'on');
    case 'run' % executing calculation
        set(findobj(gcf, 'Type', 'uicontrol'), 'Enable', 'on');
        set(findobj(gcf, 'Type', 'uicontrol', 'Style', 'popupmenu'), 'Enable', 'off');
        set(findobj(gcf, 'Type', 'uicontrol', 'Style', 'edit'), 'Enable', 'inactive');
        set(findobj(gcf, 'Type', 'uicontrol', 'Style', 'edit'), 'BackgroundColor', [0.8 0.8 0.8]);
        set(handles.start, 'String', '{}{} PAUSE');
        set(handles.start, 'Enable', 'on');
        set(handles.stop, 'Enable', 'on');
        set(handles.exit, 'Enable', 'off');
    case 'pause' % uiwait (waiting for uiresume)
        set(findobj(gcf, 'Type', 'uicontrol'), 'Enable', 'on');
        set(findobj(gcf, 'Type', 'uicontrol', 'Style', 'popupmenu'), 'Enable', 'off');
        set(findobj(gcf, 'Type', 'uicontrol', 'Style', 'edit'), 'Enable', 'inactive');
        set(findobj(gcf, 'Type', 'uicontrol', 'Style', 'edit'), 'BackgroundColor', [0.8 0.8 0.8]);
        set(handles.start, 'String', '[>RESUME');
        set(handles.start, 'Enable', 'on');
        set(handles.stop, 'Enable', 'on');
        set(handles.exit, 'Enable', 'off');
    case 'stop' % trying to stop calculation
        set(findobj(gcf, 'Type', 'uicontrol'), 'Enable', 'off');
    case 'exit' % trying to exit
        set(findobj(gcf, 'Type', 'uicontrol'), 'Enable', 'off');
    otherwise
        herr = errordlg(['Error: unknown FIGSTATE = [' ud ']']);
        uiwait(herr);
end
return;

% --------------------------------------------------------------------
function start_Callback(hObject, eventdata, handles)
ud = get(gcf, 'UserData');
switch ud
    case 'idle' % --> START
        set_figstate('run', handles);
        p = getparams(handles); % load parameters set in GUI
        if p.err
          return;
        end
        linadv(p.ijk,p.amp,p.rc,p.rnu,p.nstep,p.twait); % call calculation and drawing routines
        ud = get(gcf, 'UserData');
        if strcmp(ud, 'exit')
            delete(gcf);
            return;
        end
        set_figstate('idle', handles);
    case 'run' % --> PAUSE
        set_figstate('pause', handles);
        uiwait(gcf);
    case 'pause' % --> RESUME
        set_figstate('run', handles);
        uiresume(gcf);
    case 'stop' % --> do nothing
        return;
    case 'exit' % --> do nothing
        return;
    otherwise
        herr = errordlg(['Error: unknown FIGSTATE = [' ud ']']);
        uiwait(herr);
end
return;

% --------------------------------------------------------------------
function stop_Callback(hObject, eventdata, handles)
%set(gcf, 'Name', 'Linear Advection and Diffusion - Please wait for stopping...');
set_figstate('stop', handles);
uiresume(gcf);
return;

% --------------------------------------------------------------------
function exit_Callback(hObject, eventdata, handles)
ud = get(gcf, 'UserData');
if strcmp(ud, 'idle')
    delete(gcf);
    return;
else
    set_figstate('exit', handles);
    uiresume(gcf);
end
return;



function edit5_Callback(hObject, eventdata, handles)
% hObject    handle to edit5 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of edit5 as text
%        str2double(get(hObject,'String')) returns contents of edit5 as a double



function edit4_Callback(hObject, eventdata, handles)
% hObject    handle to edit4 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of edit4 as text
%        str2double(get(hObject,'String')) returns contents of edit4 as a double



function edit_rnu_Callback(hObject, eventdata, handles)
% hObject    handle to edit_rnu (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of edit_rnu as text
%        str2double(get(hObject,'String')) returns contents of edit_rnu as a double



function edit_rc_Callback(hObject, eventdata, handles)
% hObject    handle to edit_rc (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of edit_rc as text
%        str2double(get(hObject,'String')) returns contents of edit_rc as a double



function edit_amp_Callback(hObject, eventdata, handles)
% hObject    handle to edit_amp (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of edit_amp as text
%        str2double(get(hObject,'String')) returns contents of edit_amp as a double


