%******************************************************************************
% TITLE
%   Geophysical Fluid Dynamics - Computer Exercises -
%
%   Lorenz chaos (Lorenz, 1963)
%
% HISTORY
%   1999/06/21 Takahiro Koyama
%   1999/06/23 Shigeo Yoden
%   2005/02/01 Koichi Shin (RISH) (MATLAB code)
%
% INPUT
%   ssigma  :
%   rr      :
%   f       :
%   g       :
%   x1,x2,x3:
%******************************************************************************
function lorenz1(ssigma, rr, f, g, x1,x2,x3, nmax, nstep)

global sigma r fg
sigma = ssigma;
r = rr;
fg = f/g;

imax=3; % xyz

%nmax=100000;
%nmax=20000;

dt=0.002;
dt=0.005;
istep=1;

work = zeros(3,imax);
xp = ones(nmax,imax)*NaN;

x(1)=x1;
x(2)=x2;
x(3)=x3;
x = x';

%-- time integrations
%nstep = fix(nmax/200)
h =  graph_init(xp,0*dt);

t = 0.0;
for  n=1:nmax
  x = odrkdu(imax, 'odrkg', 'fcn1', t, t+dt, istep, x, work);
  xp(n,:) = x';

  ud = get(gcf, 'UserData');
  if strcmp(ud, 'stop') || strcmp(ud, 'exit')
    return;
  end
  if mod(n,nstep) == 0
    graph(xp,n*dt,h);
  end
  drawnow;
end

return;


%----------------------------------------------------------------------------
function h = graph_init(xp,t)

  global sigma r f g

  h.fig = gcf;
  %set(gcf,'doublebuffer','on')

  delete(get(gca,'title')); % to avoid miss-positioning in GUI
  delete(get(gca,'xlabel'));
  delete(get(gca,'ylabel'));
  delete(get(gca,'zlabel'));
  
  xymin=-35.0;xymax=35.0;
  zmin=0.0;zmax=60.0;


  title(sprintf('T=%4.3f',t))

  % axes
  plot3([xymin, xymax], [0, 0], [0, 0],'k-','LineWidth',2);
  hold on
  plot3([0, 0], [xymin, xymax], [0, 0],'k-','LineWidth',2);
  plot3([0, 0], [0, 0], [zmin, zmax],'k-','LineWidth',2);
  h.plot = plot3(xp(:,1),xp(:,2),xp(:,3),'b-');
  hold off

  axis([xymin,xymax,xymin,xymax,zmin,zmax])
  axis equal;
  grid on
  box on
  xlabel('X')
  ylabel('Y')
  zlabel('Z')

  set(gca,'xtick',[xymin,0,xymax])
  set(gca,'ytick',[xymin,0,xymax])
  set(gca,'ztick',[zmin,zmax])
  set(gca,'xtickmode','auto')
  set(gca,'xticklabelmode','auto')
  set(gca,'ytickmode','auto')
  set(gca,'yticklabelmode','auto')
  set(gca,'ztickmode','auto')
  set(gca,'zticklabelmode','auto')

 
  set(gca,'nextplot','replacechildren')
  drawnow
return


%----------------------------------------------------------------------------
function h = graph(xp, t, h)

set(h.plot,'xdata',xp(:,1),'ydata',xp(:,2),'zdata',xp(:,3))
title(sprintf('T=%4.3f',t))
drawnow

return;

%*****************************************************************************


