%******************************************************************************
% TITLE
%   Geophysical Fluid Dynamics - Computer Exercises -
%
%   Lorenz chaos (Lorenz, 1963)
%
% HISTORY
%   1999/06/21 Takahiro Koyama
%   1999/06/23 Shigeo Yoden
%******************************************************************************
function lore1(itype, r0, theta0, z0)

imax=3;
nmax=5000;

dt=0.01;
istep=1;

%real x(imax),work(3*imax),xp(nmax,imax)
x = zeros(imax,1);
work = zeros(3,imax);
xp = zeros(nmax,imax);

global sigma r f g

sigma=10.0;
r = 28.0;
f = 8.0;
g = 3.0;

%-----+--1---------2---------3---------4---------5---------6---------7-+
%-- initial conditions
t = 0.0;
x(1) = 0.0;
x(2) = 1.0;
x(3) = 0.0;

%-- time integrations
for  n=1:nmax
  %call odrkdu(imax, odrkg, fcn, t, t+dt, istep, x, work);
  x = odrkdu(imax, 'odrkg', 'fcn1', t, t+dt, istep, x, work);
  xp(n,1) = x(1);
  xp(n,2) = x(2);
  xp(n,3) = x(3);
end

%-- graphic output
%      open (1,file='expar1.data')
%      read (1,*) itype, r0, theta0, z0, cdummy
%      close(1)
      
graph(nmax, imax, xp, itype, r0, theta0, z0);

return;


%-----------------------------------------------------------------------------
function graph(nmax, imax, xp, itype, r0, theta0, z0)

xymin=-35.;xymax=35.;
zmin=0.;zmax=60.;

vxymi=-0.5;vxyma=0.5;
vzmi=0.;vzma=6.0/7.0;

kmax=540.;
eps=0.4;
period=1.0;

global sigma r f g
%-----+--1---------2---------3---------4---------5---------6---------7-+

%call slmgn(0., 0., 0., 0.1)
%print ,'(A2,f4.1)') csgi(154)//'=', sigma
%print ,'(A2,f4.1)') csgi(163)//'=', r
%print ,'(A2,I1,A1,I1)') 'b=',int(f),'/',int(g)
%ct = cttl(1) // cttl(2) // cttl(3)
%call slsttl(ct,'t', 1.0, 1.0, 0.015, 1)

% setup for itype=3
a = r0/(1. + eps)
dt = period/kmax
theta2 = pi

plot3(xp(:,1),xp(:,2),xp(:,3),'b-')
xlabel('X')
ylabel('Y')
zlabel('Z')
title(sprintf('r=%3.1f,u=%3.1f,b=%d/%d',sigma,r,f,g))
grid on

return;

for k=0:kmax
  switch itype
   case 1
    % rotation around z-axis
    theta = (theta0 - 2.*k) *pi/180.
    xeye = r0 * cos( theta )
    yeye = r0 * sin( theta )
    zeye = z0
    xobj = (xymax+xymin)/2
    yobj = xobj
    zobj = ( zmax+ zmin)/2
   
   case 2
    % straight line motion
    r = r0 - 0.5*(k-1)
    xeye = r * cos( theta0*pi/180. )
    yeye = r * sin( theta0*pi/180. )
    zeye = z0
    xobj = xeye - 100.*cos( theta0*pi/180. )
    yobj = yeye - 100.*sin( theta0*pi/180. )
    zobj = z0
  
   otherwise 3
    % Kepler-like
    theta  = 0.
    for i=0:k
      r = a *(1. - eps^2) /(1. + eps*cos(theta2) )
      theta = theta + 2.*pi*a^2 *sqrt(1.-eps^2) /(period*r^2) *dt
      theta2 = pi + theta
    end
    theta1 = theta0*pi/180. + theta
    xeye = r*sin(-theta1)
    yeye = r*cos(-theta1)
    zeye = r*cos(-theta)*z0/r0
    xobj = 0.
    yobj = 0.
    zobj = 30.
    
  end

  xeye = xeye *(vxyma-vxymi)/(xymax-xymin);
  yeye = yeye *(vxyma-vxymi)/(xymax-xymin);
  zeye = zeye *( vzma- vzmi)/( zmax- zmin);
  xobj = xobj *(vxyma-vxymi)/(xymax-xymin);
  yobj = yobj *(vxyma-vxymi)/(xymax-xymin);
  zobj = zobj *( vzma- vzmi)/( zmax- zmin);

  %%%%%
  %call grfrm
  %g3dini(xymin, xymax, xymin, xymax, zmin, zmax,...
  %       vxymi, vxyma, vxymi, vxyma, vzmi, vzma,...
  %       xeye, yeye, zeye, xobj, yobj, zobj)
  
  %call scspli(41)
  %call scplu(nmax, xp(1,1), xp(1,2), xp(1,3))
  plot3(xp(:,1),xp(:,2),xp(:,3),'k-')
  grid on
  
  drawnow
  pause
end

return
%*****************************************************************************
