%****************************************************************************
%     time-dependent solutions of Lorenz's equations    (cf. Lorenz, 1963)
%     time-integrations by Runge-Kutta-Gill
%
%     sensitive dependence on initial value
%
% HISTORY
%   1997/08/31 Shoichi Shige
%   1999/06/17 Shigeo Yoden
%
%-- experimental parameters and initial conditions
  %open (51,file='expar2.data')
  %read (51,*) sigm, r, f, g, (x(i),i=1,3), cdummy
  %close(51)
%*****************************************************************************
function lorenz2(ssigma,rr,ff,gg,x1,x2,x3, ngrmax, nstep)
  imax=3; % xyz

  %
  global sigma r fg f g
  sigma = ssigma;
  r = rr;
  f = ff;
  g = gg;
  fg = f/g;

  x = zeros(imax*2,1);
  x = [x1,x2,x3, 0,0,0]';

  %
  nmax=10000;
  nmax=5000;
  %ngrmax=10000;
  %ngrmax=5000;
  istep=1;

  dt=0.002;
  dt=0.005;
  err=1.e-3;

  %real x(2*imax),xp(0:ngrmax,2*imax),work(3*2*imax)
  %real rmserr(0:ngrmax),time(0:ngrmax)
  xp = ones(ngrmax+1,imax*2)*NaN;

  rmserr = zeros(ngrmax+1,1);
  time = zeros(ngrmax+1,1);
  
  work = zeros(imax*2,3);

  %
%  hwait = waitbar(0,'Performing computation.');

  %-- integration for nmax steps to get the attractor
  % time increment = namax*dt
  t = 0.0;
  x = odrkdu(imax, 'odrkg', 'fcn3', t, nmax*dt, nmax, x, work);

%  close(hwait);

  
  %-- restart
  x(imax+(1:imax)) = x(1:imax) + err;
  
  xp(1,:) = x';

  time = dt*(0:ngrmax);

  hfig = gcf;
  h1 = graph1_init(ngrmax, imax, xp);                    % 3D
  h2 = graph2_init(ngrmax, imax, xp, rmserr, time, err); % 2D

  for n=1:ngrmax
    x = odrkdu(2*imax, 'odrkg', 'fcn3', t, t+dt, istep, x, work);

    xp(n+1,:) = x';
  
    if mod(n,nstep) == 0
      rmserr = (xp(:,1)-xp(:,4)).^2 +(xp(:,2)-xp(:,5)).^2 +(xp(:,3)-xp(:,6)).^2;
      rmserr = sqrt(rmserr/3.0);
      rmserr(1) = err;

      drawnow;
      ud = get(gcf, 'UserData');
      if strcmp(ud, 'stop') || strcmp(ud, 'exit')
        return;
      end
      
      h1 = graph1(ngrmax, imax, xp, h1);                   % 3D
      drawnow;
      h2 = graph2(ngrmax, imax, xp, rmserr, time, err,h2); % 2D
      drawnow;
    end
  end
  


return;


%*****************************************************************************
function h1 = graph1_init(ngrmax, imax, xp)

  hax = findobj('Tag','axes_lore1');
  if isempty(hax)
    subplot(224)
  else
    set(gcf, 'CurrentAxes', hax);
    %axes(hax);
  end
  
  xymin=-30;xymax=30;
  zmin=0;zmax=60;

  h1.r = plot3(xp(:,4),xp(:,5),xp(:,6),'r-');
  hold on
  h1.b = plot3(xp(:,1),xp(:,2),xp(:,3),'b-');
  
  % axes
  plot3([xymin, xymax], [0, 0], [0, 0],'k-','LineWidth',2);
  plot3([0, 0], [xymin, xymax], [0, 0],'k-','LineWidth',2);
  plot3([0, 0], [0, 0], [zmin, zmax],'k-','LineWidth',2);
  hold off

  axis equal;
  axis([xymin,xymax,xymin,xymax,zmin,zmax])
  grid on
  box on
  xlabel('X')
  ylabel('Y')
  zlabel('Z')

  %view(10,10)  
  set(gca,'nextplot','replacechildren')
  drawnow
return

function h1 = graph1(ngrmax, imax, xp,h1)
set(h1.b,'xdata',xp(:,1),'ydata',xp(:,2),'zdata',xp(:,3))
set(h1.r,'xdata',xp(:,4),'ydata',xp(:,5),'zdata',xp(:,6))
drawnow
return

%*****************************************************************************
function h2 = graph2_init(ngrmax, imax, xp, rmserr, time, err)
  global sigma r f g

  ymin=-20.0;
  ymax=-ymin;

  %-- variavle-X
  hax = findobj('Tag','axes_lore2');
  if isempty(hax)
    subplot(411)
  else
    set(gcf, 'CurrentAxes', hax);
    %axes(hax);
  end

  h2.r = plot(time, xp(:,4),'r-');
  hold on
  h2.b = plot(time, xp(:,1),'b-');
  hold off

  axis([time(1), time(end), ymin, ymax])

  title(sprintf('\\sigma=%3.1f, \\gamma=%3.1f, b=%d/%d',sigma,r,f,g))
  xlabel('TIME');
  ylabel('X(T)');

  set(gca,'nextplot','replacechildren')
  drawnow
  
  %-- rms error
  hax = findobj('Tag','axes_lore3');
  if isempty(hax)
    subplot(412)
  else
    set(gcf, 'CurrentAxes', hax);
    %axes(hax);
  end

  h2.rms = semilogy(time, rmserr,'k-');

  axis([time(1), time(end), 0.1*err, 100.0])
  set(gca,'ytick',[0.0001,0.01,1,100])
  
  xlabel('TIME')
  ylabel('RMS ERROR')

  set(gca,'nextplot','replacechildren')
  drawnow
return

%-------------------------------------------------
function h2 = graph2(ngrmax, imax, xp, rmserr, time, err,h2)
set(h2.r,'ydata',xp(:,4))
set(h2.b,'ydata',xp(:,1))
set(h2.rms,'ydata',rmserr)
  drawnow
return

%****************************************************************************
