function varargout = lorenz2_ui(varargin)
% lorenz2_UI Application M-file for lorenz2_ui.fig
%    FIG = lorenz2_UI launch lorenz2_ui GUI.
%    lorenz2_UI('callback_name', ...) invoke the named callback.

% Last Modified by GUIDE v2.5 16-Dec-2005 16:02:16

if nargin == 0  % LAUNCH GUI

	fig = openfig(mfilename,'reuse');
	%set(fig,'Color',get(0,'defaultUicontrolBackgroundColor'));
    changeuicolor(fig)
  
	% Generate a structure of handles to pass to callbacks, and store it. 
	handles = guihandles(fig);
    
    % STARTUP code ----
    set(gcf, 'HandleVisibility', 'on');
    set(gcf, 'CloseRequestFcn', 'lorenz2_ui(''exit_Callback'',gcf,[],guidata(gcf))');
    set_figstate('idle', handles);

    [param.sigma, param.r, param.f, param.g, param.x1,param.x2,param.x3, param.label] = textread('expar2.data','%f%f%f%f%f%f%f%q%*[^\n]', 'commentstyle','matlab');
    set(handles.popup_sample,'String',param.label);
    
    handles.param = param;
  
    setparam(handles,1);
    % STARTUP code ----
	guidata(fig, handles);

	if nargout > 0
		varargout{1} = fig;
	end

else
  if ischar(varargin{1}) % INVOKE NAMED SUBFUNCTION OR CALLBACK

	try
		if (nargout)
			[varargout{1:nargout}] = feval(varargin{:}); % FEVAL switchyard
		else
			feval(varargin{:}); % FEVAL switchyard
		end
	catch
		disp(lasterr);
	end

  end
  return
  
end
return

% ------------
function setparam(handles,n)
  set(handles.edit1,'String',handles.param.sigma(n));
  set(handles.edit2,'String',handles.param.r(n));
  set(handles.edit3,'String',handles.param.f(n));
  set(handles.edit4,'String',handles.param.g(n));

  set(handles.edit5,'String',handles.param.x1(n));
  set(handles.edit6,'String',handles.param.x2(n));
  set(handles.edit7,'String',handles.param.x3(n));
return;


% --------------------------------------------------------------------
function varargout = popup_sample_Callback(h, eventdata, handles, varargin)


n = get(h,'Value');
setparam(handles,n);

return;


% --------------------------------------------------------------------
function varargout = togglebutton1_Callback(h, eventdata, handles, varargin)

cmode = 'orbit';

%cameratoolbar('show')

button_state = get(h,'Value');
if button_state == get(h,'Max') % button = push down

    %cameratoolbar('setmode',cmode)
    set(handles.togglebutton2,'Value',get(handles.togglebutton2,'Min'))
	rotate3d on
	zoom off

       
elseif button_state == get(h,'Min') % button = up

  %cmode_state = cameratoolbar('getmode');
  %if strcmp(cmode_state,cmode)
    %cameratoolbar('setmode','')
  %end  
  
  rotate3d off
  
end

return;


% --------------------------------------------------------------------
function varargout = togglebutton2_Callback(h, eventdata, handles, varargin)

cmode = 'zoom';

%cameratoolbar('show')

button_state = get(h,'Value');
if button_state == get(h,'Max') % button = push down

    %cameratoolbar('setmode',cmode)
    set(handles.togglebutton1,'Value',get(handles.togglebutton1,'Min'))
       zoom on
       rotate3d off
elseif button_state == get(h,'Min') % button = up

  %cmode_state = cameratoolbar('getmode');
  %if strcmp(cmode_state,cmode)
    %cameratoolbar('setmode','')
  %end  
  zoom off
end

return;


% --------------------------------------------------------------------
% --------------------------------------------------------------------
% --------------------------------------------------------------------
function set_figstate(newstate, handles)
set(gcf, 'UserData', newstate);
ud = get(gcf, 'UserData');
switch ud
    case 'idle' % ready to change parameters, start calculation or exit
        set(findobj(gcf, 'Type', 'uicontrol'), 'Enable', 'on');
        set(findobj(gcf, 'Type', 'uicontrol', 'Style', 'edit'), 'BackgroundColor', [1.0 1.0 1.0]);
        set(handles.start, 'String', '[> START');
        set(handles.start, 'Enable', 'on');
        set(handles.stop, 'Enable', 'off');
        set(handles.exit, 'Enable', 'on');
    case 'run' % executing calculation
        set(findobj(gcf, 'Type', 'uicontrol'), 'Enable', 'on');
        set(findobj(gcf, 'Type', 'uicontrol', 'Style', 'popupmenu'), 'Enable', 'off');
        set(findobj(gcf, 'Type', 'uicontrol', 'Style', 'edit'), 'Enable', 'inactive');
        set(findobj(gcf, 'Type', 'uicontrol', 'Style', 'edit'), 'BackgroundColor', [0.8 0.8 0.8]);
        set(handles.start, 'String', '{}{} PAUSE');
        set(handles.start, 'Enable', 'on');
        set(handles.stop, 'Enable', 'on');
        set(handles.exit, 'Enable', 'off');
    case 'pause' % uiwait (waiting for uiresume)
        set(findobj(gcf, 'Type', 'uicontrol'), 'Enable', 'on');
        set(findobj(gcf, 'Type', 'uicontrol', 'Style', 'popupmenu'), 'Enable', 'off');
        set(findobj(gcf, 'Type', 'uicontrol', 'Style', 'edit'), 'Enable', 'inactive');
        set(findobj(gcf, 'Type', 'uicontrol', 'Style', 'edit'), 'BackgroundColor', [0.8 0.8 0.8]);
        set(handles.start, 'String', '[>RESUME');
        set(handles.start, 'Enable', 'on');
        set(handles.stop, 'Enable', 'on');
        set(handles.exit, 'Enable', 'off');
    case 'stop' % trying to stop calculation
        set(findobj(gcf, 'Type', 'uicontrol'), 'Enable', 'off');
    case 'exit' % trying to exit
        set(findobj(gcf, 'Type', 'uicontrol'), 'Enable', 'off');
    otherwise
        herr = errordlg(['Error: unknown FIGSTATE = [' ud ']']);
        uiwait(herr);
end
return;

% --------------------------------------------------------------------
function start_Callback(hObject, eventdata, handles)
ud = get(gcf, 'UserData');
switch ud
    case 'idle' % --> START
        set_figstate('run', handles);
        %
        sigma = str2double(get(handles.edit1,'String'));
        r = str2double(get(handles.edit2,'String'));
        f = str2double(get(handles.edit3,'String'));
        g = str2double(get(handles.edit4,'String'));
        x1 = str2double(get(handles.edit5,'String'));
        x2 = str2double(get(handles.edit6,'String'));
        x3 = str2double(get(handles.edit7,'String'));
        nmax = str2double(get(handles.edit8,'String'));
        nstep = str2double(get(handles.edit9,'String'));
        %
        lorenz2(sigma,r,f,g,x1,x2,x3,nmax,nstep);
        %
        ud = get(gcf, 'UserData');
        if strcmp(ud, 'exit')
            delete(gcf);
            return;
        end
        set_figstate('idle', handles);
    case 'run' % --> PAUSE
        set_figstate('pause', handles);
        uiwait(gcf);
    case 'pause' % --> RESUME
        set_figstate('run', handles);
        uiresume(gcf);
    case 'stop' % --> do nothing
        return;
    case 'exit' % --> do nothing
        return;
    otherwise
        herr = errordlg(['Error: unknown FIGSTATE = [' ud ']']);
        uiwait(herr);
end
return;

% --------------------------------------------------------------------
function stop_Callback(hObject, eventdata, handles)
%set(gcf, 'Name', 'Linear Advection and Diffusion - Please wait for stopping...');
set_figstate('stop', handles);
uiresume(gcf);
return;

% --------------------------------------------------------------------
function exit_Callback(hObject, eventdata, handles)
ud = get(gcf, 'UserData');
if strcmp(ud, 'idle')
    delete(gcf);
    return;
else
    set_figstate('exit', handles);
    uiresume(gcf);
end
return;


% --------------------------------------------------------------------
function edit8_Callback(hObject, eventdata, handles)
function edit9_Callback(hObject, eventdata, handles)
