%****************************************************************************
%     time-dependent solutions of Lorenz's equations    (cf. Lorenz, 1963)
%     time-integrations by Runge-Kutta-Gill
%
%     Lorenz plot
%
% HISTORY
%   1997/08/31 Shoichi Shige
%   1999/06/17 Shigeo Yoden
%
  %-- experimental parameters
  %open (51,file='expar2.data')
  %read (51,*) sigma, r, f, g, (x(i),i=1,3), cdummy
  %close(51)
%*****************************************************************************
function lorenz3(ssigma,rr,f,g,x1,x2,x3, nmax2, nstep)

  imax=3;

  nmax=150000;
  nmax=2000;
  mmax=50000;

  dt=0.001;
  dt=0.002;
  istep=1;

  global sigma r fg
  sigma = ssigma;
  r = rr;
  fg = f/g;

  x = [x1,x2,x3]';
      
  %real x(imax),work(3*imax),zp(nmax/10)
  work = zeros(imax,3);
  zp = zeros(nmax/10,1);

  %-- initial conditions -------------------------------------------
  t    = 0.0;
  z2 = -999.0;
  z3 = -999.0;
  
  %-- integration for nmax steps to get the attractor
  %hwait = waitbar(0,'Step 1: Wait...');
  hwait = waitbar(0,'Step 1: Performing computation.');

  waitbarstep = fix(nmax/20);
  
  % time increment = namax*dt
  t = 0.0;
  x = odrkdu(imax, 'odrkg', 'fcn4', t, nmax*dt, nmax, x, work);
  
  % time increment = namax*dt
  for n=1:nmax
    x = odrkdu(imax, 'odrkg', 'fcn4', t, t+dt, istep, x, work);
    z1 = z2;
    z2 = z3;
    z3= x(3);
    if (z1<z2) & (z2>z3)
      zpm = z2;
    end
    if mod(n,waitbarstep) == 0
      waitbar(n/nmax, hwait);
    end
  end
  close(hwait);


  %-- restart -------------------------------------------------------
  t = 0.0;
  z = zeros(nmax2+2,1);
  zpn = zeros(mmax,1);
  zpn(1) = zpm;

  h = graphics_init(zpn,sigma,r,f,g);
  
  z(1)=-999;
  z(2)=-999;
  
  m = 1;
  for n=1:nmax2
    x = odrkdu(imax, 'odrkg', 'fcn4', t, t+dt, istep, x, work);

    z(n+2)= x(3);

    drawnow;
    
    if (z(n)<z(n+1)) & (z(n+1)>z(n+2))
      m = m+1;
      zpn(m) = z(n+1);
    end
    
    ud = get(gcf, 'UserData');
    if strcmp(ud, 'stop') || strcmp(ud, 'exit')
      return;
    end
    drawnow;
    if mod(n,nstep) == 0
      h = graphics(zpn,h);
    end
  end
  

return;
%*****************************************************************************

%-- graphics
function h = graphics_init(zpn, sigma,r,f,g)

  h.fig = gcf;
  plot(1) %dummy
  h.plot = plot(zpn(1:(end-1)), zpn(2:end),'k.');
  xlabel('Z_n');
  ylabel('Z_n+1');
  
  axis equal
  axis([25,50,25,50])
  title(sprintf('\\sigma=%3.1f, r=%3.1f, b=%d/%d',sigma,r,f,g))
drawnow

return

function h = graphics(zpn,h)

set(h.plot,'xdata',zpn(1:(end-1)),'ydata',zpn(2:end))
drawnow

return;
  
