%*-----------------------------------------------------------------------
%*     Runge-Kutta-Gill algorithm routine.
%*                                                 Oct. 5, 1990  S.Sakai
%*-----------------------------------------------------------------------
%*     Copyright (C) 2000 GFD Dennou Club. All rights reserved.
%*-----------------------------------------------------------------------
function xout = odrkg(n, fcn, t, dt, x, dx, xout, work)

    dt2 = dt/2.0;
    %r2 = sqrt(2.0);
    r2 = 1.41421356237310;
      
    c1 =  (1.0 - 1.0/r2)*dt;
    c2 =  -2.0 + 3.0/r2;
    c3 =   2.0 -    r2;
    c4 =  (1.0 + 1.0/r2)*dt;
    c5 =  (2.0 + 3.0/r2)/3.0;
    c6 = -(2.0 +    r2)/3.0;

    %--- 1st step ----------------------------------
    xout = x + dt2*dx;

    %--- 2nd step ----------------------------------
    tt = t+dt2;
    work = feval(fcn, n, tt, xout, work); % call
    
    xout = xout  + c1*(work-dx);
    dx   = c2*dx + c3*work;
      
    %--- 3rd step ----------------------------------
    work = feval(fcn, n, tt, xout, work); % call
      
    xout = xout  + c4*(work-dx);
    dx   = c5*dx + c6*work;

    %--- 4th step ----------------------------------
    tt = t+dt;
    work = feval(fcn, n, tt, xout, work); % call
    
    xout = xout + dt*(work/6.0+dx);

return
