%*-----------------------------------------------------------------------
%*     Runge-Kutta-Gill algorithm routine.
%*                                                 Oct. 5, 1990  S.Sakai
%*-----------------------------------------------------------------------
%*     Copyright (C) 2000 GFD Dennou Club. All rights reserved.
%*-----------------------------------------------------------------------
%      2004/05/24 Koichi Shin (RISH) (convert to the MATLAB code)
%***********************************************************************
function xout = odrkg(n, fcn, t, dt, x, dx, xout, work)
      %dimension  x(n), dx(n), xout(n), work(n)

      dt2 = dt/2.;
      c1 =  (1. - 1./sqrt(2.))*dt;
      c2 =  -2. + 3./sqrt(2.);
      c3 =   2. -    sqrt(2.);
      c4 =  (1. + 1./sqrt(2.))*dt;
      c5 =  (2. + 3./sqrt(2.))/3.;
      c6 = -(2. +    sqrt(2.))/3.;

%*--------------------------- 1st step ----------------------------------

      i=1:n;
      xout(i) = x(i) + dt2*dx(i);

%*--------------------------- 2nd step ----------------------------------

      tt = t+dt2;
      %call fcn(n, tt, xout, work)
      work = feval(fcn, n, tt, xout, work);
      
      i=1:n;
      xout(i) = xout(i)  + c1*(work(i)-dx(i));
      dx(i)   = c2*dx(i) + c3*work(i);
      

%*--------------------------- 3rd step ----------------------------------

      %call fcn(n, tt, xout, work)
      work = feval(fcn, n, tt, xout, work);
      
      i=1:n;
      xout(i) = xout(i)  + c4*(work(i)-dx(i));
      dx(i)   = c5*dx(i) + c6*work(i);


%*--------------------------- 4th step ----------------------------------

      tt = t+dt;
      %call fcn(n, tt, xout, work)
      work = feval(fcn, n, tt, xout, work);
      
      i=1:n;
      xout(i) = xout(i) + dt*(work(i)/6.+dx(i));

return
