%******************************************************************************
% TITLE
%   Geophysical Fluid Dynamics - Computer Exercises -
%
%   Dynamics of vortex filament system:
%   Induced flow by one or two vortex filaments
%
% HISTORY
%   1999/04/29 Shigeo Yoden
%   2004/05/25 Koichi Shin (RISH) (convert to MATLAB code)
%   2005/12/10 Katsushi Sato (alter GUI)
%
% INPUT
%   gamma(1), xvort(1), yvort(1),   % circulation and x,y-coordinates of vortex filemant 1
%   gamma(2), xvort(2), yvort(2),   % circulation and x,y-coordinates of vortex filemant 2
%******************************************************************************
function ptvort1flow(gamma1,xvort1,yvort1,gamma2,xvort2,yvort2)

gamma= [gamma1,gamma2];
xvort= [xvort1, xvort2];
yvort= [yvort1,yvort2];

nmax=2; % nmax: number of vortex filament

xmin=-10.;xmax=10.;ymin=-10.;ymax=10.;
[xx,yy] = meshgrid(xmin:xmax,ymin:ymax);

ix=21;
jy=21;

%real gamma(nmax),xvort(nmax),yvort(nmax)
u = zeros(ix,jy);
v = zeros(ix,jy);

%call glrget( 'rmiss', rmiss )
rmiss = 0;


%***************************
% initialize graphics
figure(gcf);
cla;


%***************************
% flow field
for n=1:(nmax+1)
  ud = get(gcf, 'UserData');
  if strcmp(ud, 'stop') || strcmp(ud, 'exit')
    return;
  end

  if n <= nmax

    %***************************
    % flow field induced by each vortex filament
    delx = (xmax-xmin)/(ix-1);
    dely = (ymax-ymin)/(jy-1);
    for j=1:jy
      for i=1:ix
        x = xmin + delx*(i-1);
        y = ymin + dely*(j-1);
        if i==1 | i==ix | j==1 | j==jy
          u(i,j) = rmiss;
          v(i,j) = rmiss;
        else
          rr = (xvort(n)-x)^2 + (yvort(n)-y)^2;
          if rr ~= 0
            u(i,j) =   (0.5*gamma(n)/pi)*(yvort(n)-y) / rr;
            v(i,j) = - (0.5*gamma(n)/pi)*(xvort(n)-x) / rr;
          else
            u(i,j) = rmiss;
            v(i,j) = rmiss;
          end
        end
      end
    end

    %***************************
    % plot
    linestyle = {'r.' 'b.'};
    colorstyle = {'m' 'c'};

    quiver(xx',yy',u,v,char(colorstyle(n)));
    hold on
    plot(xvort(n), yvort(n),char(linestyle(n)));
    hold off

  else

    %***************************
    % flow field induced by the vortex filament system
    delx = (xmax-xmin)/(ix-1);
    dely = (ymax-ymin)/(jy-1);
    for j=1:jy
      for i=1:ix
        x = xmin + delx*(i-1);
        y = ymin + dely*(j-1);
        if i==1 | i==ix | j==1 | j==jy
          u(i,j) = rmiss;
          v(i,j) = rmiss;
        else
          u(i,j) = 0.0;
          v(i,j) = 0.0;
          for nn=(1):(nmax)
            rr =  (xvort(nn)-x)^2 + (yvort(nn)-y)^2;
            if rr ~= 0
              u(i,j) = u(i,j) +(0.5*gamma(nn)/pi)*(yvort(nn)-y) / rr;
              v(i,j) = v(i,j) -(0.5*gamma(nn)/pi)*(xvort(nn)-x) / rr;
            end
          end  
        end
      end
    end

    %***************************
    % plot
  quiver(xx',yy',u,v,'g')
  hold on
    plot(xvort(1), yvort(1),'r.')
    plot(xvort(2), yvort(2),'b.')
  hold off
  
  end

  axis equal
  axis([xmin, xmax, ymin, ymax]);
  xlabel('X');
  ylabel('Y');
  title('Vortex filaments')

  set(gcf,'WindowButtonUpFcn','pushanykey')
  set(gcf,'KeyPressFcn','pushanykey')
  if n <= nmax
    uiwait(gcf)
    %pause(2)
  end

end

return
%*****************************************************************************
