%******************************************************************************
% TITLE
%   Geophysical Fluid Dynamics - Computer Exercises -
%
%   Dynamics of vortex filament system:
%   Motion of two-vortex-filaments system
%
% HISTORY
%   1997/09/05 Kazuhiro Ando (1st grade of master course, 
%                             Severe Storm and Atmospheric Environment, 
%                             Division of Atmospheric and Hydrospheric Disasters, DPRI)
%   1999/04/28 Shigeo Yoden
%   2004/05/25 Koichi Shin (RISH) (convert to MATLAB code)
%   2005/12/10 Katsushi Sato (alter GUI)
%
% INPUT
%   gamma(1), xvort(1), yvort(1),   % circulation and x,y-coordinates of vortex filemant 1
%   gamma(2), xvort(2), yvort(2),   % circulation and x,y-coordinates of vortex filemant 2
%******************************************************************************
function  ptvort1motion(gamma1, x1, y1, gamma2, x2, y2, dt, itmax)

global xmin xmax ymin ymax
global gma

nmax=2;
%itmax=2000; % nmax: number of vortex filament, itmax: number of steps in integration

%dt=0.1;
%dt=0.25;
istep=1;     % dt: temporal grid size(=tend-t), istep: number of step

xmin=-10.;xmax=10.;ymin=-10.;ymax=10.;

work = zeros(2*nmax,3);

x = zeros(itmax+1, nmax);
y = zeros(itmax+1, nmax);

x(0+1,:)=[x1,x2];
y(0+1,:)=[y1,y2];

gma=[gamma1,gamma2];


% initial values
t = 0.0;
z = [x(0+1,1); y(0+1,1); x(0+1,2); y(0+1,2)];

h = graphics_init(x,y);

% temporal development
for ii=1:itmax
  ud = get(gcf, 'UserData');
  if strcmp(ud, 'stop') || strcmp(ud, 'exit')
    return;
  end

  tend =t + dt;

  %call odrkdu(2*nmax, odrkg, fcn, t, tend, istep, z, work)
  z = odrkdu(2*nmax, 'odrkg', 'fcn', t, tend, istep, z, work);

  x(ii+1,1) = z(1);
  y(ii+1,1) = z(2);
  x(ii+1,2) = z(3);
  y(ii+1,2) = z(4);

  h = graphics(x,y,ii,h);
  drawnow;
end

return


%------------------------
%
function h = graphics_init(x,y)

global xmin xmax ymin ymax

h.fig = gcf;

plot(x(1,1),y(1,1),'c.','MarkerSize',4) % dummy plot

h.r = line(x(1,1),y(1,1), 'color','c', 'Marker','.', ...
           'erase','none','MarkerSize',4);

h.b = line(x(1,2),y(1,2), 'color','y', 'Marker','.', ...
           'erase','none','MarkerSize',4);

h.rdot = line(x(1,1),y(1,1), 'color','r', 'Marker','.', ...
           'erase','xor','MarkerSize',20);

h.bdot = line(x(1,2),y(1,2), 'color','b', 'Marker','.', ...
           'erase','xor','MarkerSize',20);



axis equal
axis([xmin, xmax, ymin, ymax])
xlabel('X')
ylabel('Y')
h.title = title('Vortex filaments');
box on

set(h.fig,'DoubleBuffer','on');

return

%------------------------
% visualization of calculation result
function h = graphics(x,y,n,h)

set(h.r,'xdata',x(n,1),'ydata',y(n,1));
set(h.b,'xdata',x(n,2),'ydata',y(n,2));

set(h.rdot,'xdata',x(n,1),'ydata',y(n,1));
set(h.bdot,'xdata',x(n,2),'ydata',y(n,2));

return
%****************************************************************************
