%******************************************************************************
% TITLE
%   Geophysical Fluid Dynamics - Computer Exercises -
%
%   Dynamics of vortex filament system:
%   Temporal development of two-vortex-filaments system
%
% HISTORY
%   1997/09/05 Kazuhiro Ando (1st grade of master course, 
%                             Severe Storm and Atmospheric Environment, 
%                             Division of Atmospheric and Hydrospheric Disasters, DPRI)
%   1999/04/28 Shigeo Yoden
%   2004/05/25 Koichi Shin (RISH) (convert to MATLAB code)
%
% INPUT
%   gamma(1), xvort(1), yvort(1),   % circulation and x,y-coordinates of vortex filemant 1
%   gamma(2), xvort(2), yvort(2),   % circulation and x,y-coordinates of vortex filemant 2
%   gamma(3), xvort(3), yvort(3),   % circulation and x,y-coordinates of vortex filemant 3
%******************************************************************************
function ptvort2(gamma1,x1,y1, gamma2,x2,y2, gamma3,x3,y3, dt, itmax)

global gma
gma=[gamma1, gamma2, gamma3];

nmax=3;
%itmax=50000; % nmax: number of vortex filament, itmax: number of steps in integration 

%dt=5;
istep=1;     % dt: temporal grid size(=tend-t), istep: number of time steps

xmin=-70.;xmax=70.;ymin=-70.;ymax=70.;
work = zeros(2*nmax,3);

%***************************
% initial values
t = 0.0;
z = [x1; y1; x2; y2; x3; y3];

%***************************
% initialize graphics
hfig = figure(gcf);
%set(gca,'NextPlot','replacechildren')
set(gca,'Drawmode','fast')

plot(x1,y1) % dummy plot
%get(gca)

reddot = line(x1,y1,'color','r', 'Marker','.', 'MarkerSize',20 ...
           ,'erase','xor');

bluedot = line(x2,y2,'color','b', 'Marker','.', 'MarkerSize',20 ...
           ,'erase','xor');

greendot = line(x3,y3,'color','g', 'Marker','.', 'MarkerSize',20 ...
           ,'erase','xor');

red = line(x1,y1,'color','c', 'Marker','.', 'MarkerSize',4, ...
           'erase','none');

blue = line(x2,y2,'color','y', 'Marker','.', 'MarkerSize',4, ...
           'erase','none');

green = line(x3,y3,'color','m', 'Marker','.', 'MarkerSize',4, ...
           'erase','none');

axis equal
axis([xmin, xmax, ymin, ymax])
set(gca,'Box','on');
htitle = title(sprintf('Point Vortices'));
xlabel('X')
ylabel('Y')

%***************************
% time evolution
tend =t + dt;
for ii=1:itmax
  ud = get(gcf, 'UserData');
  if strcmp(ud, 'stop') || strcmp(ud, 'exit')
    return;
  end
  
  z = odrkdu(2*nmax, 'odrkg', 'fcn3', t, tend, istep, z, work);

  drawnow
  
  %***************************
  % visualization of calculation result
  set(reddot,'xdata',z(1),'ydata',z(2));
  set(bluedot,'xdata',z(3),'ydata',z(4));
  set(greendot,'xdata',z(5),'ydata',z(6));
  set(red,'xdata',z(1),'ydata',z(2));
  set(blue,'xdata',z(3),'ydata',z(4));
  set(green,'xdata',z(5),'ydata',z(6));

  if mod(ii,100) == 0
    %set(htitle,'String',sprintf('Vortex filaments: Time=%5.1f',ii*dt));
    set(gcf,'Name',sprintf('Vortex filaments: Time=%5.1f',ii*dt));
  end

  drawnow
end

return
%*****************************************************************************
