function varargout = ptvort2_ui(varargin)
% ptvort2_UI Application M-file for ptvort2_ui.fig
%    FIG = ptvort2_UI launch ptvort2_ui GUI.
%    ptvort2_UI('callback_name', ...) invoke the named callback.

% Last Modified by GUIDE v2.5 15-Dec-2005 10:07:35

if nargin == 0  % LAUNCH GUI

  fig = openfig(mfilename,'reuse');
  changeuicolor(fig)

  % Generate a structure of handles to pass to callbacks, and store it. 
  handles = guihandles(fig);

  % STARTUP code ----
  set(gcf, 'HandleVisibility', 'on');
  set(gcf, 'CloseRequestFcn', 'ptvort2_ui(''exit_Callback'',gcf,[],guidata(gcf))');
  set_figstate('idle', handles);
  
  [param.g1, param.x1, param.y1, ...
   param.g2, param.x2, param.y2, ...
   param.g3, param.x3, param.y3, ...
   param.dt, param.itmax, ...0
   param.label] = textread('expar2.data','%f%f%f%f%f%f%f%f%f%f%f%q%*[^\n]', 'commentstyle','matlab');
  set(handles.popup_sample,'String',param.label);
  
  handles.param = param;
  
  setparam(handles,1);
  % STARTUP code ----

  guidata(fig, handles);

  if nargout > 0
    varargout{1} = fig;
  end

elseif ischar(varargin{1}) % INVOKE NAMED SUBFUNCTION OR CALLBACK

  try
    if (nargout)
      [varargout{1:nargout}] = feval(varargin{:}); % FEVAL switchyard
    else
      feval(varargin{:}); % FEVAL switchyard
    end
  catch
    disp(lasterr);
  end

end


% --------------------------------------------------------------------
function setparam(handles,n)
  set(handles.edit1,'String',handles.param.g1(n));
  set(handles.edit2,'String',handles.param.x1(n));
  set(handles.edit3,'String',handles.param.y1(n));

  set(handles.edit4,'String',handles.param.g2(n));
  set(handles.edit5,'String',handles.param.x2(n));
  set(handles.edit6,'String',handles.param.y2(n));

  set(handles.edit7,'String',handles.param.g3(n));
  set(handles.edit8,'String',handles.param.x3(n));
  set(handles.edit9,'String',handles.param.y3(n));
  
  set(handles.edit10,'String',handles.param.dt(n));
  set(handles.edit11,'String',handles.param.itmax(n));
return;


% --------------------------------------------------------------------
function varargout = popup_sample_Callback(h, eventdata, handles, varargin)

n = get(h,'Value');
setparam(handles,n);

return;


% --------------------------------------------------------------------
% --------------------------------------------------------------------
% --------------------------------------------------------------------
function set_figstate(newstate, handles)
set(gcf, 'UserData', newstate);
ud = get(gcf, 'UserData');
switch ud
    case 'idle' % ready to change parameters, start calculation or exit
        set(findobj(gcf, 'Type', 'uicontrol'), 'Enable', 'on');
        set(findobj(gcf, 'Type', 'uicontrol', 'Style', 'edit'), 'BackgroundColor', [1.0 1.0 1.0]);
        set(handles.start, 'String', '[> START');
        set(handles.start, 'Enable', 'on');
        set(handles.stop, 'Enable', 'off');
        set(handles.exit, 'Enable', 'on');
    case 'run' % executing calculation
        set(findobj(gcf, 'Type', 'uicontrol'), 'Enable', 'on');
        set(findobj(gcf, 'Type', 'uicontrol', 'Style', 'popupmenu'), 'Enable', 'off');
        set(findobj(gcf, 'Type', 'uicontrol', 'Style', 'edit'), 'Enable', 'inactive');
        set(findobj(gcf, 'Type', 'uicontrol', 'Style', 'edit'), 'BackgroundColor', [0.8 0.8 0.8]);
        set(handles.start, 'String', '{}{} PAUSE');
        set(handles.start, 'Enable', 'on');
        set(handles.stop, 'Enable', 'on');
        set(handles.exit, 'Enable', 'off');
    case 'pause' % uiwait (waiting for uiresume)
        set(findobj(gcf, 'Type', 'uicontrol'), 'Enable', 'on');
        set(findobj(gcf, 'Type', 'uicontrol', 'Style', 'popupmenu'), 'Enable', 'off');
        set(findobj(gcf, 'Type', 'uicontrol', 'Style', 'edit'), 'Enable', 'inactive');
        set(findobj(gcf, 'Type', 'uicontrol', 'Style', 'edit'), 'BackgroundColor', [0.8 0.8 0.8]);
        set(handles.start, 'String', '[>RESUME');
        set(handles.start, 'Enable', 'on');
        set(handles.stop, 'Enable', 'on');
        set(handles.exit, 'Enable', 'off');
    case 'stop' % trying to stop calculation
        set(findobj(gcf, 'Type', 'uicontrol'), 'Enable', 'off');
    case 'exit' % trying to exit
        set(findobj(gcf, 'Type', 'uicontrol'), 'Enable', 'off');
    otherwise
        herr = errordlg(['Error: unknown FIGSTATE = [' ud ']']);
        uiwait(herr);
end
return;

% --------------------------------------------------------------------
function start_Callback(hObject, eventdata, handles)
ud = get(gcf, 'UserData');
switch ud
    case 'idle' % --> START
        set_figstate('run', handles);
        %
        gamma1 = str2double(get(handles.edit1,'String'));
        x1 = str2double(get(handles.edit2,'String'));
        y1 = str2double(get(handles.edit3,'String'));
        gamma2 = str2double(get(handles.edit4,'String'));
        x2 = str2double(get(handles.edit5,'String'));
        y2 = str2double(get(handles.edit6,'String'));
        gamma3 = str2double(get(handles.edit7,'String'));
        x3 = str2double(get(handles.edit8,'String'));
        y3 = str2double(get(handles.edit9,'String'));
        dt = str2double(get(handles.edit10,'String'));
        itmax = str2double(get(handles.edit11,'String'));
        ptvort2(gamma1,x1,y1,gamma2,x2,y2,gamma3,x3,y3,dt,itmax);
        %
        ud = get(gcf, 'UserData');
        if strcmp(ud, 'exit')
            delete(gcf);
            return;
        end
        set_figstate('idle', handles);
    case 'run' % --> PAUSE
        set_figstate('pause', handles);
        uiwait(gcf);
    case 'pause' % --> RESUME
        set_figstate('run', handles);
        uiresume(gcf);
    case 'stop' % --> do nothing
        return;
    case 'exit' % --> do nothing
        return;
    otherwise
        herr = errordlg(['Error: unknown FIGSTATE = [' ud ']']);
        uiwait(herr);
end
return;

% --------------------------------------------------------------------
function stop_Callback(hObject, eventdata, handles)
%set(gcf, 'Name', 'Linear Advection and Diffusion - Please wait for stopping...');
set_figstate('stop', handles);
uiresume(gcf);
return;

% --------------------------------------------------------------------
function exit_Callback(hObject, eventdata, handles)
ud = get(gcf, 'UserData');
if strcmp(ud, 'idle')
    delete(gcf);
    return;
else
    set_figstate('exit', handles);
    uiresume(gcf);
end
return;



% --------------------------------------------------------------------
function edit1_Callback(hObject, eventdata, handles)
function edit2_Callback(hObject, eventdata, handles)
function edit3_Callback(hObject, eventdata, handles)
function edit4_Callback(hObject, eventdata, handles)
function edit5_Callback(hObject, eventdata, handles)
function edit6_Callback(hObject, eventdata, handles)
function edit7_Callback(hObject, eventdata, handles)
function edit8_Callback(hObject, eventdata, handles)
function edit9_Callback(hObject, eventdata, handles)
function edit10_Callback(hObject, eventdata, handles)
function varargout = edit11_Callback(h, eventdata, handles, varargin)
function varargout = edit12_Callback(h, eventdata, handles, varargin)


