%******************************************************************************
% TITLE
%   Geophysical Fluid Dynamics - Computer Exercises -
%
%   Parcel Method (1): Static stability
%
% HISTORY
%   1999/05/20 Tomonori Ohmiya
%   1999/05/22 Shigeo Yoden
%   2004/05/29 Koichi Shin (RISH) (convert to MATLAB code)
%
% INPUT
%   z0, w0, drdz(1), drdz(2),   % initial conditions and
%   drdz(3), drdz(4), drdz(5)   % vertical gradient of density
%******************************************************************************
function stasta(z0,w0,drdz1,drdz2,drdz3,drdz4,drdz5, dt, itmax)

%real drdz(mmax)   % density gradient
drdz = [drdz1,drdz2,drdz3,drdz4,drdz5];

g=9.8;
rho0=1.0;    % rho0: density of parcel

ntone=69;
idlev=1; % number of color, initial index of color, increment

nx=2;
nz=ntone+1;
xmin=0.;xmax=1.;
zmin=-1.0;zmax=1.0;
mmax=5;

%itmax=200;
%dt=0.2;

% contour
rho = zeros(nx,nz,mmax);

% position of parcel
x = zeros(1,itmax+1);
z = zeros(itmax+1,mmax);

%complex qai,qa(mmax),qb(mmax),qn(mmax)
qai = i;
rmiss = 0.0;

%****** density field ******
for m=1:mmax
  for jj=1:nz
    zz = zmin + (zmax-zmin)*(jj-1)/(nz-1);
    if drdz(m) == 0.0
      rho(:,jj,m) = rho0 + 1.e-5*zz;
    else
      rho(:,jj,m) = rho0 + drdz(m)*zz;
    end
  end
end



%******* tone *******
drzmin = min(drdz);
drzmax = max(drdz);

prz = 0.7;
if abs(drzmin) >= abs(drzmax)
  for m=1:mmax
    if drdz(m) == drzmin
      prmin = rho0 + prz*drdz(m);
      prdel = abs(2.*prz*drdz(m))/ntone;
      mmm = m;
    end
  end
else
  for m=1:mmax
    if drdz(m) == drzmax
      prmin = rho0 - prz*drdz(m);
      prdel =     2.*prz*drdz(m)/ntone;
      mmm = m;
    end
  end
end


%**** initial condition, Brunt-Vaisala frequency and etc. ****
t = 0.;
x(0+1) = (xmax-xmin)/2;
for m=1:mmax
  z(0+1,m) = z0;
  if drdz(m) == 0.;
    qn(m) = rmiss;
  else
    if drdz(m) < 0.;
      qn(m) = sqrt( -g*drdz(m)/rho0 );
      qa(m) = 0.5*( z0 - qai*w0/qn(m) );
      qb(m) = 0.5*( z0 + qai*w0/qn(m) );
    else
      qn(m) = sqrt(  g*drdz(m)/rho0 );
      qa(m) = 0.5*( z0 + w0/qn(m) );
      qb(m) = 0.5*( z0 - w0/qn(m) );
    end  
  end
end

%****** calculation ******
for ii=1:itmax
  t = ii*dt;
  x(ii+1) = x(0+1);
  for m=1:mmax
    if drdz(m) < 0.0;
      z(ii+1,m) = qb(m)*exp(-qai*qn(m)*t);
      % +real(  qa(m)*exp( qai*qn(m)*t) 
    else
      if drdz(m) > 0.0
        z(ii+1,m) = qa(m)*exp(qn(m)*t) + qb(m)*exp(-qn(m)*t);
      else
        z(ii+1,m) = z0 + w0*t;
      end
    end
  end
end



%************************************************************
% graph 

% initialization
hfig = figure(gcf);
%set(hfig,'UserData',1);
set(hfig,'BackingStore','on');
set(hfig,'DoubleBuffer','on');
udjet = flipud(jet);
colormap(udjet);

%****** mark color
%jj = 1:nz;
%zz = zmin + (zmax-zmin)*(jj-1)/(nz-1)
%markcolor = (rho0+1.0e-5*zz)';
  % Temporarily half. Strictly, it must be done in the range of caxis. 
  % <---- What does this comment mean?? (by K. Sato)
markcolor = udjet(length(udjet)/2,:);


% Erase axes in figure, unless the UserData exists (for GUI).
h = get(gcf,'Children');
for l = 1:length(h)
  if strcmp(get(h(l),'Type') ,'axes')
    if isempty(get(h(l),'UserData'))
      delete(h(l))
    end
  end
end

%
mainaxes = gca;
set(mainaxes,'Visible','off');
apos = get(mainaxes,'Position');

%vvxmin = 0.12;
%vvxmax = 0.85;
%vvxdel = (vvxmax - vvxmin)/mmax;

yyy = linspace(zmin,zmax,nz);
xxx = linspace(xmin,xmax,nx);


%********* graph m **********
for m=1:mmax
  haxes(m) = axes('Position',[apos(1)+apos(3)/(mmax+1)*(m-1), apos(2), ...
                      apos(3)/(mmax+1)*0.8, apos(4)]); %left bottom w h
  
  rrr = squeeze(rho(:,:,m));
  pcolor(xxx,yyy,rrr')
  caxis([0.9,1.1])
  shading flat

  axis([ xmin,  xmax,  zmin,  zmax]);
  set(gca,'TickDir','out')
  set(gca,'XTick',[])
  set(gca,'XTickLabel','')
  set(gca,'YTickLabel','')

  ctitle = sprintf('%6.3f', drdz(m));
  title(ctitle)

  if m == 1
    ylabel('HEIGHT');
    ctitle = sprintf('dr/dz=%6.3f', drdz(m));
    title(ctitle);
    set(gca,'YTick',-1:0.2:1);
    set(gca,'YTickLabel','-1|||||0|||||1');
  end

  %***********
  hold on
  ho(m) = plot(real(x(1)), real(z(1,m)),'ko','MarkerSize',12,'MarkerFaceColor',markcolor);
  hold off
end

%********* color bar **********c
hcbar = axes('Position',[apos(1)+apos(3)/(mmax+1)*mmax, apos(2), ...
                      apos(3)/(mmax+1)*0.5, apos(4)]); %left bottom w h

rrr = squeeze(rho(:,:,mmm));
pcolor(xxx,yyy,rrr')
shading flat
set(gca,'TickDir','out');
set(gca,'XTick',[]);
set(gca,'XTickLabel','');
set(gca,'YTick',-1:0.2:1);
set(gca,'YTickLabel','1.1|||||1|||||0.9');
set(gca,'YAxisLocation','Right');
title('r','Fontname','Symbol','FontSize',12)
  

%********* draw loop **********c
for ii=0:itmax
  ud = get(gcf, 'UserData');
  if strcmp(ud, 'stop') || strcmp(ud, 'exit')
    return;
  end
  
  for m=1:mmax
    haxes(m);
    set(ho(m), 'xdata',real(x(ii+1)),'ydata', real(z(ii+1,m)));

    if m == 1
      ctime = sprintf('Time=%4.1f',ii*dt);
      xlabel(ctime)
    end
  end

  pause(0.01)
end

return;
%*****************************************************************************
