function varargout = stasta_ui(varargin)
% STASTA_UI Application M-file for stasta_ui.fig
%    FIG = STASTA_UI launch stasta_ui GUI.
%    STASTA_UI('callback_name', ...) invoke the named callback.

% Last Modified by GUIDE v2.5 11-Dec-2005 01:41:10

if nargin == 0  % LAUNCH GUI

	fig = openfig(mfilename,'reuse');
	%set(fig,'Color',get(0,'defaultUicontrolBackgroundColor'));
    changeuicolor(fig)

	% Generate a structure of handles to pass to callbacks, and store it. 
	handles = guihandles(fig);

    % STARTUP code ----
    set(gcf, 'HandleVisibility', 'on');
    set(gcf, 'CloseRequestFcn', 'stasta_ui(''exit_Callback'',gcf,[],guidata(gcf))');
    set_figstate('idle', handles);
    
    [param.z0, param.w0, param.drdz1, param.drdz2, param.drdz3, param.drdz4, param.drdz5, param.label] = textread('expar1.data','%f%f%f%f%f%f%f%q%*[^\n]', 'commentstyle','matlab');
    set(handles.popup_sample,'String',param.label);
  
    handles.param = param;
  
    setparam(handles,1);
    % STARTUP code ----
    
    guidata(fig, handles);

	if nargout > 0
		varargout{1} = fig;
	end

else
  if ischar(varargin{1}) % INVOKE NAMED SUBFUNCTION OR CALLBACK

	try
		if (nargout)
			[varargout{1:nargout}] = feval(varargin{:}); % FEVAL switchyard
		else
			feval(varargin{:}); % FEVAL switchyard
		end
	catch
		disp(lasterr);
	end

  end
    return

end
return

function setparam(handles,n)
  set(handles.edit_z0,'String',handles.param.z0(n));
  set(handles.edit_w0,'String',handles.param.w0(n));
  set(handles.edit1,'String',handles.param.drdz1(n));
  set(handles.edit2,'String',handles.param.drdz2(n));
  set(handles.edit3,'String',handles.param.drdz3(n));
  set(handles.edit4,'String',handles.param.drdz4(n));
  set(handles.edit5,'String',handles.param.drdz5(n));
return;


% --------------------------------------------------------------------
function varargout = popup_sample_Callback(h, eventdata, handles, varargin)

n = get(h,'Value');
setparam(handles,n);

return;


% --------------------------------------------------------------------
% --------------------------------------------------------------------
% --------------------------------------------------------------------
function set_figstate(newstate, handles)
set(gcf, 'UserData', newstate);
ud = get(gcf, 'UserData');
switch ud
    case 'idle' % ready to change parameters, start calculation or exit
        set(findobj(gcf, 'Type', 'uicontrol'), 'Enable', 'on');
        set(findobj(gcf, 'Type', 'uicontrol', 'Style', 'edit'), 'BackgroundColor', [1.0 1.0 1.0]);
        set(handles.start, 'String', '[> START');
        set(handles.start, 'Enable', 'on');
        set(handles.stop, 'Enable', 'off');
        set(handles.exit, 'Enable', 'on');
    case 'run' % executing calculation
        set(findobj(gcf, 'Type', 'uicontrol'), 'Enable', 'on');
        set(findobj(gcf, 'Type', 'uicontrol', 'Style', 'popupmenu'), 'Enable', 'off');
        set(findobj(gcf, 'Type', 'uicontrol', 'Style', 'edit'), 'Enable', 'inactive');
        set(findobj(gcf, 'Type', 'uicontrol', 'Style', 'edit'), 'BackgroundColor', [0.8 0.8 0.8]);
        set(handles.start, 'String', '{}{} PAUSE');
        set(handles.start, 'Enable', 'on');
        set(handles.stop, 'Enable', 'on');
        set(handles.exit, 'Enable', 'off');
    case 'pause' % uiwait (waiting for uiresume)
        set(findobj(gcf, 'Type', 'uicontrol'), 'Enable', 'on');
        set(findobj(gcf, 'Type', 'uicontrol', 'Style', 'popupmenu'), 'Enable', 'off');
        set(findobj(gcf, 'Type', 'uicontrol', 'Style', 'edit'), 'Enable', 'inactive');
        set(findobj(gcf, 'Type', 'uicontrol', 'Style', 'edit'), 'BackgroundColor', [0.8 0.8 0.8]);
        set(handles.start, 'String', '[>RESUME');
        set(handles.start, 'Enable', 'on');
        set(handles.stop, 'Enable', 'on');
        set(handles.exit, 'Enable', 'off');
    case 'stop' % trying to stop calculation
        set(findobj(gcf, 'Type', 'uicontrol'), 'Enable', 'off');
    case 'exit' % trying to exit
        set(findobj(gcf, 'Type', 'uicontrol'), 'Enable', 'off');
    otherwise
        herr = errordlg(['Error: unknown FIGSTATE = [' ud ']']);
        uiwait(herr);
end
return;

% --------------------------------------------------------------------
function start_Callback(hObject, eventdata, handles)
ud = get(gcf, 'UserData');
switch ud
    case 'idle' % --> START
        set_figstate('run', handles);
        %
        z0 = str2double(get(handles.edit_z0,'String'));
        w0 = str2double(get(handles.edit_w0,'String'));
        d1 = str2double(get(handles.edit1,'String'));
        d2 = str2double(get(handles.edit2,'String'));
        d3 = str2double(get(handles.edit3,'String'));
        d4 = str2double(get(handles.edit4,'String'));
        d5 = str2double(get(handles.edit5,'String'));
        dt = str2double(get(handles.edit6,'String'));
        nmax = str2double(get(handles.edit7,'String'));
        if isnan(z0)|isnan(w0)|isnan(d1)|isnan(d2)|isnan(d3)|isnan(d4)|isnan(d5)
          errordlg('Input parameters must be numerical characters.');
          return;
        end
        %
        stasta(z0,w0,d1,d2,d3,d4,d5,dt,nmax);
        %
        ud = get(gcf, 'UserData');
        if strcmp(ud, 'exit')
            delete(gcf);
            return;
        end
        set_figstate('idle', handles);
    case 'run' % --> PAUSE
        set_figstate('pause', handles);
        uiwait(gcf);
    case 'pause' % --> RESUME
        set_figstate('run', handles);
        uiresume(gcf);
    case 'stop' % --> do nothing
        return;
    case 'exit' % --> do nothing
        return;
    otherwise
        herr = errordlg(['Error: unknown FIGSTATE = [' ud ']']);
        uiwait(herr);
end
return;

% --------------------------------------------------------------------
function stop_Callback(hObject, eventdata, handles)
%set(gcf, 'Name', 'Linear Advection and Diffusion - Please wait for stopping...');
set_figstate('stop', handles);
uiresume(gcf);
return;

% --------------------------------------------------------------------
function exit_Callback(hObject, eventdata, handles)
ud = get(gcf, 'UserData');
if strcmp(ud, 'idle')
    delete(gcf);
    return;
else
    set_figstate('exit', handles);
    uiresume(gcf);
end
return;
